# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import llvm

from numba import *
from numba import nodes
from numba.typesystem import is_obj, promote_to_native
from numba.codegen.codeutils import llvm_alloca, if_badval
from numba.codegen import debug


class ObjectCoercer(object):
    """
    Object that knows how to convert to/from objects using Py_BuildValue
    and PyArg_ParseTuple.
    """

    # TODO: do all of this in a specializer

    type_to_buildvalue_str = {
        char: "b",
        short: "h",
        int_: "i",
        long_: "l",
        longlong: "L",
        Py_ssize_t: "n",
        npy_intp: "n", # ?
        size_t: "n", # ?
        uchar: "B",
        ushort: "H",
        uint: "I",
        ulong: "k",
        ulonglong: "K",

        float_: "f",
        double: "d",
        complex128: "D",

        object_: "O",
        bool_: "b", # ?
        c_string_type: "s",
        char.pointer() : "s",
    }

    def __init__(self, translator):
        self.context = translator.context
        self.translator = translator
        self.builder = translator.builder
        self.llvm_module = self.builder.basic_block.function.module
        sig, self.py_buildvalue = self.context.external_library.declare(
            self.llvm_module, 'Py_BuildValue')
        sig, self.pyarg_parsetuple = self.context.external_library.declare(
            self.llvm_module, 'PyArg_ParseTuple')
        sig, self.pyerr_clear = self.context.external_library.declare(
            self.llvm_module, 'PyErr_Clear')
        self.function_cache = translator.function_cache
        self.NULL = self.translator.visit(nodes.NULL_obj)

    def check_err(self, llvm_result, callback=None, cmp=llvm.core.ICMP_EQ,
                  pos_node=None):
        """
        Check for errors. If the result is NULL, and error should have been set
        Jumps to translator.error_label if an exception occurred.
        """
        assert llvm_result.type.kind == llvm.core.TYPE_POINTER, llvm_result.type
        int_result = self.translator.builder.ptrtoint(llvm_result,
                                                      llvm_types._intp)
        NULL = llvm.core.Constant.int(int_result.type, 0)

        if callback:
            if_badval(self.translator, int_result, NULL,
                      callback=callback or default_callback, cmp=cmp)
        else:
            test = self.builder.icmp(cmp, int_result, NULL)
            name = 'no_error'
            if hasattr(pos_node, 'lineno'):
                name = 'no_error_%s' % error.format_pos(pos_node).rstrip(": ")
            bb = self.translator.append_basic_block(name)
            self.builder.cbranch(test, self.translator.error_label, bb)
            self.builder.position_at_end(bb)

        return llvm_result

    def check_err_int(self, llvm_result, badval):
        llvm_badval = llvm.core.Constant.int(llvm_result.type, badval)
        if_badval(self.translator, llvm_result, llvm_badval,
                  callback=lambda b, *args: b.branch(self.translator.error_label))

    def _create_llvm_string(self, str):
        return self.translator.visit(nodes.ConstNode(str, c_string_type))

    def lstr(self, types, fmt=None):
        "Get an llvm format string for the given types"
        typestrs = []
        result_types = []
        for type in types:
            if is_obj(type):
                type = object_
            elif type.is_int:
                type = promote_to_native(type)

            result_types.append(type)
            typestrs.append(self.type_to_buildvalue_str[type])

        str = "".join(typestrs)
        if fmt is not None:
            str = fmt % str

        if debug.debug_conversion:
            self.translator.puts("fmt: %s" % str)

        result = self._create_llvm_string(str)
        return result_types, result

    def buildvalue(self, types, *largs, **kwds):
        # The caller should check for errors using check_err or by wrapping
        # its node in an ObjectTempNode
        name = kwds.get('name', '')
        fmt = kwds.get('fmt', None)
        types, lstr = self.lstr(types, fmt)
        largs = (lstr,) + largs

        if debug.debug_conversion:
            self.translator.puts("building... %s" % name)
            # func_type = object_(*types).pointer()
        # py_buildvalue = self.builder.bitcast(
        #         self.py_buildvalue, func_type.to_llvm(self.context))
        py_buildvalue = self.py_buildvalue
        result = self.builder.call(py_buildvalue, largs, name=name)

        if debug.debug_conversion:
            self.translator.puts("done building... %s" % name)
            nodes.print_llvm(self.translator.env, object_, result)
            self.translator.puts("--------------------------")

        return result

    def npy_intp_to_py_ssize_t(self, llvm_result, type):
    #        if type == minitypes.npy_intp:
    #            lpy_ssize_t = minitypes.Py_ssize_t.to_llvm(self.context)
    #            llvm_result = self.translator.caster.cast(llvm_result, lpy_ssize_t)
    #            type = minitypes.Py_ssize_t

        return llvm_result, type

    def py_ssize_t_to_npy_intp(self, llvm_result, type):
    #        if type == minitypes.npy_intp:
    #            lnpy_intp = minitypes.npy_intp.to_llvm(self.context)
    #            llvm_result = self.translator.caster.cast(llvm_result, lnpy_intp)
    #            type = minitypes.Py_ssize_t

        return llvm_result, type

    def convert_single_struct(self, llvm_result, type):
        types = []
        largs = []
        for i, (field_name, field_type) in enumerate(type.fields):
            types.extend((c_string_type, field_type))
            largs.append(self._create_llvm_string(field_name))
            struct_attr = self.builder.extract_value(llvm_result, i)
            largs.append(struct_attr)

        return self.buildvalue(types, *largs, name='struct', fmt="{%s}")

    def convert_single(self, type, llvm_result, name=''):
        "Generate code to convert an LLVM value to a Python object"
        llvm_result, type = self.npy_intp_to_py_ssize_t(llvm_result, type)
        if type.is_struct:
            return self.convert_single_struct(llvm_result, type)
        elif type.is_complex:
            # We have a Py_complex value, construct a Py_complex * temporary
            new_result = llvm_alloca(self.translator.lfunc, self.builder,
                                     llvm_result.type, name='complex_temp')
            self.builder.store(llvm_result, new_result)
            llvm_result = new_result

        return self.buildvalue([type], llvm_result, name=name)

    def build_tuple(self, types, llvm_values):
        "Build a tuple from a bunch of LLVM values"
        assert len(types) == len(llvm_values)
        return self.buildvalue(lstr, *llvm_values, name='tuple', fmt="(%s)")

    def build_list(self, types, llvm_values):
        "Build a tuple from a bunch of LLVM values"
        assert len(types) == len(llvm_values)
        return self.buildvalue(types, *llvm_values, name='list',  fmt="[%s]")

    def build_dict(self, key_types, value_types, llvm_keys, llvm_values):
        "Build a dict from a bunch of LLVM values"
        types = []
        largs = []
        for k, v, llvm_key, llvm_value in zip(key_types, value_types,
                                              llvm_keys, llvm_values):
            types.append(k)
            types.append(v)
            largs.append(llvm_key)
            largs.append(llvm_value)

        return self.buildvalue(types, *largs, name='dict', fmt="{%s}")

    def parse_tuple(self, lstr, llvm_tuple, types, name=''):
        "Unpack a Python tuple into typed llvm variables"
        lresults = []
        for i, type in enumerate(types):
            var = llvm_alloca(self.translator.lfunc, self.builder,
                              type.to_llvm(self.context),
                              name=name and "%s%d" % (name, i))
            lresults.append(var)

        largs = [llvm_tuple, lstr] + lresults

        if debug.debug_conversion:
            self.translator.puts("parsing tuple... %s" % (types,))
            nodes.print_llvm(self.translator.env, object_, llvm_tuple)

        parse_result = self.builder.call(self.pyarg_parsetuple, largs)
        self.check_err_int(parse_result, 0)

        # Some conversion functions don't reset the exception state...
        # self.builder.call(self.pyerr_clear, [])

        if debug.debug_conversion:
            self.translator.puts("successfully parsed tuple...")

        return [self.builder.load(result) for result in lresults]

    def to_native(self, type, llvm_tuple, name=''):
        "Generate code to convert a Python object to an LLVM value"
        types, lstr = self.lstr([type])
        lresult, = self.parse_tuple(lstr, llvm_tuple, [type], name=name)
        return lresult



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

from numba import jit
from numpy import zeros

import unittest

@jit()
def test():
    foo = zeros((1,))
    foo[0] = 0

@jit()
def test2():
    foo = [0]
    foo[0] = 0

class TestIssue50(unittest.TestCase):
    def test_1d_arr_setitem(self):
        self.assertEquals(test(), None)

    def test_list_setitem(self):
        self.assertEqual(test2(), None)

if __name__ == "__main__":
    unittest.main()



from numba import *

jitv = jit(void(), warnstyle='simple') #, nopython=True)




def simple_return():
    """
    >>> result = jitv(simple_return)
    Warning 14:4: Unreachable code
    """
    return
    print('Where am I?')

def simple_loops():
    """
    >>> result = jitv(simple_loops)
    Warning 28:8: Unreachable code
    Warning 32:8: Unreachable code
    Warning 36:8: Unreachable code
    Warning 41:12: Unreachable code
    Warning 46:8: Unreachable code
    Warning 50:4: Unreachable code
    """
    for i in range(10):
        continue
        print('Never be here')

    while True:
        break
        print('Never be here')

    while True:
        break
        print('Never be here')

    for i in range(10):
        for j in range(10):
            return
            print("unreachable")
        else:
            print("unreachable")
        print("unreachable")
        return
        print("unreachable")

    print("unreachable")
    return
    print("unreachable")

def conditional(a, b):
    if a:
        return 1
    elif b:
        return 2
    else:
        return 37
    print('oops')

if __name__ == "__main__":
#    jitv(simple_loops)
#    jitv(simple_return)
    import numba
    numba.testing.testmod()



import unittest

import numpy as np
from numba import autojit 

mu = 0.1
Lx, Ly = 101, 101

@autojit 
def diffusionObstacleStep(u,tempU,iterNum):
    for n in range(iterNum):
        for i in range(1, Lx - 1):
            for j in range(1, Ly - 1):
                u[i,j] = mu * (tempU[i+1,j]-2*tempU[i,j]+tempU[i-1,j] +
                               tempU[i,j+1]-2*tempU[i,j]+tempU[i,j-1])

        # Bug in Meta??
        # tempU, u = u, tempU
        # -> Assign(targets=[Name(id='tempU', ctx=Store()),
        #                    Name(id='u', ctx=Store())],
        #           value=Name(id='u', ctx=Load()))
        temp = u
        u = tempU
        tempU = temp

def get_arrays():
    u = np.zeros([Lx, Ly], dtype=np.float64)
    tempU = np.zeros([Lx, Ly], dtype=np.float64)
    u[Lx / 2, Ly / 2] = 1000.0
    return tempU, u

def test_diffusion():
    tempU, u = get_arrays()
    iterNum = 10
    diffusionObstacleStep(u, tempU, iterNum)

    tempU_numpy, u_numpy = get_arrays()
    diffusionObstacleStep.py_func(u_numpy, tempU_numpy, iterNum)

    print(u)
    print(u_numpy)
    assert np.allclose(u, u_numpy)

if __name__ == "__main__":
    test_diffusion()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import numba as nb
from numba import *
from numba import nodes
from numba import typesystem
from numba.typesystem import get_type

import numpy as np

GROW = 2

def notimplemented(msg):
    raise NotImplementedError("'%s' method" % msg)

def container_methods(item_type, notimplemented):
    # NOTE: numba will use the global 'notimplemented' function, not the
    # one passed in :(

    @item_type(Py_ssize_t) # TODO: slicing!
    def getitem(self, key):
        if not (0 <= key < self.size):
            # TODO: Implement raise !
            # raise IndexError(key)
            [][key] # tee hee

        return self.buf[key]

    @void(Py_ssize_t, item_type) # TODO: slice assignment!
    def setitem(self, key, value):
        if not (0 <= key < self.size):
            # TODO: Implement raise !
            # raise IndexError(key)
            [][key]

        self.buf[key] = value

    @void(item_type)
    def append(self, value):
        size = self.size
        if size >= self.buf.shape[0]:
            # NOTE: initial bufsize must be greater than zero
            self.buf.resize(int(size * GROW), refcheck=False)

        self.buf[size] = value
        self.size = size + 1

    @void(object_)
    def extend(self, iterable):
        # TODO: something fast for common cases (e.g. typedlist,
        #                                        np.ndarray, etc)
        for obj in iterable:
            self.append(obj)

    @Py_ssize_t(item_type)
    def index(self, value):
        # TODO: comparison of complex numbers (#121)
        buf = self.buf
        for i in range(self.size):
             if buf[i] == value:
                 return i

        [].index(value) # raise ValueError

    @Py_ssize_t(item_type)
    def count(self, value):
        # TODO: comparison of complex numbers (#121)
        count = 0
        buf = self.buf
        for i in range(self.size):
             # TODO: promotion of (bool_, int_)
             if buf[i] == value:
                 count += 1

        return count

    return locals()

#-----------------------------------------------------------------------
# Infer types for typed containers (typedlist, typedtuple)
#-----------------------------------------------------------------------

def typedcontainer_infer(compile_typedcontainer, type_node, iterable_node):
    """
    Type inferer for typed containers, register with numba.register_inferer().

    :param compile_typedcontainer: item_type -> typed container extension class
    :param type_node: type parameter to typed container constructor
    :param iterable_node: value parameter to typed container constructor (optional)
    """
    assert type_node is not None

    type = get_type(type_node)
    if type.is_cast:
        elem_type = type.dst_type

        # Pre-compile typed list implementation
        typedcontainer_ctor = compile_typedcontainer(elem_type)

        # Inject the typedlist directly to avoid runtime implementation lookup
        iterable_node = iterable_node or nodes.const(None, object_)
        result = nodes.call_pyfunc(typedcontainer_ctor, (iterable_node,))
        return nodes.CoercionNode(result, typedcontainer_ctor.exttype)

    return object_



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import ast

from numba import templating
from numba import error, pipeline, nodes, ufunc_builder
from numba.minivect import specializers, miniast, miniutils
from numba import utils, functions
from numba import typesystem
from numba import visitors

from numba.support.numpy_support import slicenodes
from numba.vectorize import basic

print_ufunc = False


def is_elementwise_assignment(context, assmnt_node):
    target_type = assmnt_node.targets[0].type
    value_type = assmnt_node.value.type

    if target_type.is_array:
        # Allow arrays and scalars
        context.promote_types(target_type, value_type)
        return not value_type.is_object

    return False


class ArrayExpressionRewrite(visitors.NumbaTransformer):
    """
    Find element-wise expressions and run ElementalMapper to turn it into
    a minivect AST or a ufunc.
    """

    nesting_level = 0
    elementwise = False

    is_slice_assign = False

    def register_array_expression(self, node, lhs=None):
        """
        Start the mapping process for the outermost node in the array expression.
        """
        self.elementwise = False

    def visit_elementwise(self, elementwise, node):
        if elementwise and self.nesting_level == 0:
            return self.register_array_expression(node)

        self.nesting_level += 1
        self.generic_visit(node)
        self.nesting_level -= 1
        self.elementwise = elementwise
        return node

    def get_py_ufunc_ast(self, lhs, node):
        if lhs is not None:
            lhs.ctx = ast.Load()

        builder = ufunc_builder.UFuncConverter()
        tree = builder.visit(node)
        ufunc_ast = builder.build_ufunc_ast(tree)

        if print_ufunc:
            from meta import asttools

            module = ast.Module(body=[ufunc_ast])
            print((asttools.python_source(module)))

        # Vectorize Python function
        if lhs is None:
            restype = node.type
        else:
            restype = lhs.type.dtype

        argtypes = [op.type.dtype if op.type.is_array else op.type
                        for op in builder.operands]
        signature = restype(*argtypes)

        return ufunc_ast, signature, builder

    def get_py_ufunc(self, lhs, node):
        ufunc_ast, signature, ufunc_builder = self.get_py_ufunc_ast(lhs, node)
        py_ufunc = ufunc_builder.compile_to_pyfunc(ufunc_ast)
        return py_ufunc, signature, ufunc_builder

    def visit_Assign(self, node):
        self.is_slice_assign = False
        self.visitlist(node.targets)

        target_node = node.targets[0]
        is_slice_assign = self.is_slice_assign

        self.nesting_level = self.is_slice_assign
        node.value = self.visit(node.value)
        self.nesting_level = 0

        elementwise = self.elementwise

        if (len(node.targets) == 1 and is_slice_assign and
                is_elementwise_assignment(self.context, node)):
            target_node = slicenodes.rewrite_slice(target_node,
                                                      self.nopython)
            return self.register_array_expression(node.value, lhs=target_node)

        return node

    def visit_Subscript(self, node):
        # print ast.dump(node)
        self.generic_visit(node)
        is_store = isinstance(node.ctx, ast.Store)
        self.is_slice_assign = is_store and node.type.is_array

        if is_store:
            if nodes.is_ellipsis(node.slice):
                return node.value
        elif node.value.type.is_array and node.type.is_array:
            node = slicenodes.rewrite_slice(node, self.nopython)

        return node

    def visit_MathNode(self, node):
        elementwise = node.arg.type.is_array
        return self.visit_elementwise(elementwise, node)

    def visit_BinOp(self, node):
        elementwise = node.type.is_array
        return self.visit_elementwise(elementwise, node)

    visit_UnaryOp = visit_BinOp


class ArrayExpressionRewriteUfunc(ArrayExpressionRewrite):
    """
    Compile array expressions to ufuncs. Then call the ufunc with the array
    arguments.

        vectorizer_cls: the ufunc vectorizer to use

    CANNOT be used in a nopython context
    """

    def __init__(self, context, func, ast, vectorizer_cls=None):
        super(ArrayExpressionRewriteUfunc, self).__init__(context, func, ast)
        self.vectorizer_cls = vectorizer_cls or basic.BasicASTVectorize

    def register_array_expression(self, node, lhs=None):
        super(ArrayExpressionRewriteUfunc, self).register_array_expression(node,
                                                                           lhs)
        py_ufunc, signature, ufunc_builder = self.get_py_ufunc(lhs, node)

        # Vectorize Python function
        vectorizer = self.vectorizer_cls(py_ufunc)
        vectorizer.add(restype=signature.return_type, argtypes=signature.args)
        ufunc = vectorizer.build_ufunc()

        # Call ufunc
        args = ufunc_builder.operands
        if lhs is None:
            keywords = None
        else:
            keywords = [ast.keyword('out', lhs)]

        func = nodes.ObjectInjectNode(ufunc)
        call_ufunc = nodes.ObjectCallNode(signature=None, func=func, args=args,
                                          keywords=keywords, py_func=ufunc)
        return nodes.ObjectTempNode(call_ufunc)


class NumbaproStaticArgsContext(utils.NumbaContext):
    "Use a static argument list: shape, data1, strides1, data2, strides2, ..."

    astbuilder_cls = miniast.ASTBuilder
    # debug = True
    # debug_elements = True


class ArrayExpressionRewriteNative(ArrayExpressionRewrite):
    """
    Compile array expressions to a minivect kernel that calls a Numba
    scalar kernel with scalar inputs:

        a[:, :] = b[:, :] * c[:, :]

    becomes

        tmp_a = slice(a)
        tmp_b = slice(b)
        tmp_c = slice(c)
        shape = broadcast(tmp_a, tmp_b, tmp_c)
        call minikernel(shape, tmp_a.data, tmp_a.strides,
                               tmp_b.data, tmp_b.strides,
                               tmp_c.data, tmp_c.strides)

    with

        def numba_kernel(b, c):
            return b * c

        def minikernel(...):
            for (...)
                for(...)
                    a[i, j] = numba_kernel(b[i, j], c[i, j])

    CAN be used in a nopython context
    """

    def array_attr(self, node, attr):
        # Perform a low-level bitcast from object to an array type
        # array = nodes.CoercionNode(node, float_[:])
        array = node
        return nodes.ArrayAttributeNode(attr, array)

    def register_array_expression(self, node, lhs=None):
        super(ArrayExpressionRewriteNative, self).register_array_expression(
            node, lhs)

        lhs_type = lhs.type if lhs else node.type
        is_expr = lhs is None

        if node.type.is_array and lhs_type.ndim < node.type.ndim:
            # TODO: this is valid in NumPy if the leading dimensions of the
            # TODO: RHS have extent 1
            raise error.NumbaError(
                node, "Right hand side must have a "
                      "dimensionality <= %d" % lhs_type.ndim)

        # Create ufunc scalar kernel
        ufunc_ast, signature, ufunc_builder = self.get_py_ufunc_ast(lhs, node)
        signature.struct_by_reference = True

        # Compile ufunc scalar kernel with numba
        ast.fix_missing_locations(ufunc_ast)
        func_env, (_, _, _) = pipeline.run_pipeline2(
            self.env, None, ufunc_ast, signature,
            function_globals={},
        )

        # Manual linking
        lfunc = func_env.lfunc

        # print lfunc
        operands = ufunc_builder.operands
        functions.keep_alive(self.func, lfunc)

        operands = [nodes.CloneableNode(operand) for operand in operands]

        if lhs is not None:
            lhs = nodes.CloneableNode(lhs)
            broadcast_operands = [lhs] + operands
            lhs = lhs.clone
        else:
            broadcast_operands = operands[:]

        shape = slicenodes.BroadcastNode(lhs_type, broadcast_operands)
        operands = [op.clone for op in operands]

        if lhs is None and self.nopython:
            raise error.NumbaError(
                node, "Cannot allocate new memory in nopython context")
        elif lhs is None:
            # TODO: determine best output order at runtime
            shape = shape.cloneable
            lhs = nodes.ArrayNewEmptyNode(lhs_type, shape.clone,
                                          lhs_type.is_f_contig).cloneable

        # Build minivect wrapper kernel
        context = NumbaproStaticArgsContext()
        context.llvm_module = self.env.llvm_context.module
        # context.debug = True
        context.optimize_broadcasting = False
        b = context.astbuilder

        variables = [b.variable(name_node.type, "op%d" % i)
                     for i, name_node in enumerate([lhs] + operands)]
        miniargs = [b.funcarg(variable) for variable in variables]
        body = miniutils.build_kernel_call(lfunc.name, signature, miniargs, b)

        minikernel = b.function_from_numpy(
            templating.temp_name("array_expression"), body, miniargs)
        lminikernel, ctypes_kernel = context.run_simple(
            minikernel, specializers.StridedSpecializer)

        # Build call to minivect kernel
        operands.insert(0, lhs)
        args = [shape]
        scalar_args = []
        for operand in operands:
            if operand.type.is_array:
                data_p = self.array_attr(operand, 'data')
                data_p = nodes.CoercionNode(data_p,
                                            operand.type.dtype.pointer())
                if not isinstance(operand, nodes.CloneNode):
                    operand = nodes.CloneNode(operand)
                strides_p = self.array_attr(operand, 'strides')
                args.extend((data_p, strides_p))
            else:
                scalar_args.append(operand)

        args.extend(scalar_args)
        result = nodes.NativeCallNode(minikernel.type, args, lminikernel)

        # Use native slicing in array expressions
        slicenodes.mark_nopython(ast.Suite(body=result.args))

        if not is_expr:
            # a[:] = b[:] * c[:]
            return result

        # b[:] * c[:], return new array as expression
        return nodes.ExpressionNode(stmts=[result], expr=lhs.clone)



import sys

import numba

from numba.testing.test_support import autojit_py3doc
# NOTE: See also issues/test_issue_56

@autojit_py3doc
def test_bitwise_and(a, b):
    """
    >>> test_bitwise_and(0b01, 0b10)
    0L
    >>> test_bitwise_and(0b01, 0b11)
    1L

    >>> test_bitwise_and(0b01, 2.0)
    Traceback (most recent call last):
        ...
    NumbaError: 27:15: Expected an int, or object, or bool

    >>> test_bitwise_and(2.0, 0b01)
    Traceback (most recent call last):
        ...
    NumbaError: 27:11: Expected an int, or object, or bool

    """
    return a & b

@autojit_py3doc
def test_bitwise_or(a, b):
    """
    >>> test_bitwise_or(0b00, 0b00)
    0L
    >>> test_bitwise_or(0b00, 0b01)
    1L
    >>> test_bitwise_or(0b10, 0b00)
    2L
    >>> test_bitwise_or(0b01, 0b10)
    3L
    >>> test_bitwise_or(0b01, 0b11)
    3L

    >>> test_bitwise_or(0b01, 2.0)
    Traceback (most recent call last):
        ...
    NumbaError: 54:15: Expected an int, or object, or bool

    >>> test_bitwise_or(2.0, 0b01)
    Traceback (most recent call last):
        ...
    NumbaError: 54:11: Expected an int, or object, or bool

    """
    return a | b


@autojit_py3doc
def test_bitwise_xor(a, b):
    """
    >>> test_bitwise_xor(0b00, 0b00)
    0L
    >>> test_bitwise_xor(0b00, 0b01)
    1L
    >>> test_bitwise_xor(0b10, 0b00)
    2L
    >>> test_bitwise_xor(0b01, 0b10)
    3L
    >>> test_bitwise_xor(0b01, 0b11)
    2L

    >>> test_bitwise_xor(0b01, 2.0)
    Traceback (most recent call last):
        ...
    NumbaError: 82:15: Expected an int, or object, or bool

    >>> test_bitwise_xor(2.0, 0b01)
    Traceback (most recent call last):
        ...
    NumbaError: 82:11: Expected an int, or object, or bool

    """
    return a ^ b

@autojit_py3doc
def test_shift_left(a, b):
    """
    >>> test_shift_left(5, 2)
    20L
    >>> test_shift_left(-5, 2)
    -20L
    """
    return a << b

@autojit_py3doc
def test_shift_right(a, b):
    """
    >>> test_shift_right(20, 2)
    5L
    >>> test_shift_right(-20, 2)
    -5L
    """
    return a >> b

@autojit_py3doc
def test_invert(a):
    """
    >>> test_invert(5)
    -6L
    >>> test_invert(-5)
    4L
    """
    return ~a


numba.testing.testmod()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import os
import sys
import shutil
import subprocess
from fnmatch import fnmatchcase
from distutils.util import convert_path

try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

import numpy

# import numba
import gen_type_conversion

from Cython.Distutils import build_ext
from Cython.Distutils.extension import Extension as CythonExtension

if sys.version_info[:2] < (2, 6):
    raise Exception('numba requires Python 2.6 or greater.')

import versioneer

#------------------------------------------------------------------------
# Setup constants and arguments
#------------------------------------------------------------------------

versioneer.versionfile_source = 'numba/_version.py'
versioneer.versionfile_build = 'numba/_version.py'
versioneer.tag_prefix = ''
versioneer.parentdir_prefix = 'numba-'

cmdclass = versioneer.get_cmdclass()
cmdclass['build_ext'] = build_ext

setup_args = {
    'long_description': open('README.md').read(),
}


numba_root = os.path.dirname(os.path.abspath(__file__))
deps_root = os.path.join(numba_root, 'deps')
pyext_root = os.path.join(deps_root, 'pyextensibletype')
pyext_dst = os.path.join(numba_root, "numba", "pyextensibletype")

def get_include():
    """Use numba.get_include() instead (make numba importable without
    building it first)
    """
    return os.path.join(numba_root, "numba", "include")

numba_include_dir = get_include()

#------------------------------------------------------------------------
# Package finding
#------------------------------------------------------------------------

def find_packages(where='.', exclude=()):
    out = []
    stack=[(convert_path(where), '')]
    while stack:
        where, prefix = stack.pop(0)
        for name in os.listdir(where):
            fn = os.path.join(where,name)
            if ('.' not in name and os.path.isdir(fn) and
                os.path.isfile(os.path.join(fn, '__init__.py'))
            ):
                out.append(prefix+name)
                stack.append((fn, prefix+name+'.'))

    if sys.version_info[0] == 3:
        exclude = exclude + ('*py2only*', )

    for pat in list(exclude) + ['ez_setup', 'distribute_setup']:
        out = [item for item in out if not fnmatchcase(item, pat)]
    return out

#------------------------------------------------------------------------
# 2to3
#------------------------------------------------------------------------

def run_2to3():
    import lib2to3.refactor
    from distutils.command.build_py import build_py_2to3 as build_py
    print("Installing 2to3 fixers")
    # need to convert sources to Py3 on installation
    fixes = 'dict imports imports2 unicode ' \
            'xrange itertools itertools_imports long types'.split()
    fixes = ['lib2to3.fixes.fix_' + fix 
             for fix in fixes]
    build_py.fixer_names = fixes
    cmdclass["build_py"] = build_py
    # cmdclass["build"] = build_py

    # Distribute options
    # setup_args["use_2to3"] = True

#------------------------------------------------------------------------
# pyextensibletype
#------------------------------------------------------------------------

def cleanup_pyextensibletype():
    if os.path.exists(pyext_dst):
        shutil.rmtree(pyext_dst)

def register_pyextensibletype():
    with open(os.path.join(deps_root, '__init__.py'), 'w'):
        pass
    with open(os.path.join(pyext_root, '__init__.py'), 'w'):
        pass

    shutil.copytree(pyext_root, pyext_dst)

    from deps.pyextensibletype import setupconfig
    exts = setupconfig.get_extensions(pyext_dst, "numba.pyextensibletype")

    return exts

#------------------------------------------------------------------------
# Generate code for build
#------------------------------------------------------------------------

build = set(sys.argv) & set(['build', 'build_ext', 'install'])
cleanup_pyextensibletype()

if build:
    gen_type_conversion.run()
    # TODO: Finish and release pyextensibletype
    extensibletype_extensions = register_pyextensibletype()
else:
    extensibletype_extensions = []

extensibletype_include = "numba/pyextensibletype/include"

if sys.version_info[0] >= 3:
    run_2to3()

#------------------------------------------------------------------------
# setup
#------------------------------------------------------------------------

setup(
    name="numba",
    version=versioneer.get_version(),
    author="Continuum Analytics, Inc.",
    author_email="numba-users@continuum.io",
    url="http://numba.github.com",
    license="BSD",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        # "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        # "Programming Language :: Python :: 3.2",
        "Topic :: Utilities",
    ],
    description="compiling Python code using LLVM",
    packages=find_packages(),
    entry_points = {
        'console_scripts': [
            'pycc = numba.pycc:main',
            ]
    },
    package_data={
        '': ['*.md'],
        'numba.minivect': ['include/*'],
        'numba.asdl.common': ['*.asdl'],
        'numba.asdl.py2_7': ['*.asdl'],
        'numba.asdl.py3_2': ['*.asdl'],
        'numba.asdl.py3_3': ['*.asdl'],
        'numba.external.utilities': ['*.c', '*.h'],
        'numba': ['*.c', '*.h', 'include/*', '*.pxd'],
        'numba.vectorize': ['*.h'],
    },
    ext_modules=extensibletype_extensions + [
        Extension(
            name="numba.vectorize._internal",
            sources=["numba/vectorize/_internal.c",
                     "numba/vectorize/_ufunc.c",
                     "numba/vectorize/_gufunc.c"],
            include_dirs=[numpy.get_include(), "numba/minivect/include/"],
            depends=["numba/vectorize/_internal.h",
                     "numba/minivect/include/miniutils.h"]),

        Extension(
            name="numba.external.utilities.utilities",
            sources=["numba/external/utilities/utilities.c"],
            include_dirs=[numba_include_dir, extensibletype_include],
            depends=["numba/external/utilities/type_conversion.c",
                     "numba/external/utilities/virtuallookup.c",
                     "numba/external/utilities/generated_conversions.c",
                     "numba/external/utilities/generated_conversions.h"]),
        CythonExtension(
            name="numba.pyconsts",
            sources=["numba/pyconsts.pyx"],
            depends=["numba/_pyconsts.pxd"],
            include_dirs=[numba_include_dir]),
        CythonExtension(
            name="numba.exttypes.extension_types",
            sources=["numba/exttypes/extension_types.pyx"],
            cython_gdb=True),
        CythonExtension(
            name="numba.numbawrapper",
            sources=["numba/numbawrapper.pyx", "numba/numbafunction.c"],
            depends=["numba/numbafunction.h"],
            include_dirs=[numba_include_dir,
                          numpy.get_include()],
            cython_gdb=True),
    ],
    cmdclass=cmdclass,
    **setup_args
)



import os
import ctypes
import doctest

from numba import *
import numba

try:
    import cffi
    ffi = cffi.FFI()
except ImportError:
    ffi = None

# ______________________________________________________________________

def test_cffi_calls():
    if ffi is not None:
        make_cffi_calls()

# ______________________________________________________________________
# Tests

@autojit(nopython=True)
def call_cffi_func(func, value):
    return func(value)

def make_cffi_calls():
    # Test printf for nopython and no segfault
    ffi.cdef("int printf(char *, ...);", override=True)
    lib = ffi.dlopen(None)
    printf = lib.printf
    call_cffi_func(printf, "Hello world!\n")

# ______________________________________________________________________

if __name__ == "__main__":
    test_cffi_calls()



#! /usr/bin/env python
# ______________________________________________________________________
'''test_multiarray_api

Test the code generation utility class numba.multiarray_api.MultiarrayAPI.
'''
# ______________________________________________________________________

import ctypes

import llvm.core as lc
import llvm.ee as le

from numba.llvm_types import _int32, _intp, _intp_star, _void_star, _numpy_array, _head_len
import numba.multiarray_api as ma

import numpy as np

import unittest

import logging
logging.basicConfig(level=logging.DEBUG)

# ______________________________________________________________________

_pyobj_to_pyobj = ctypes.CFUNCTYPE(ctypes.py_object, ctypes.py_object)


_void_star = lc.Type.pointer(lc.Type.int(8))

_make_const_int = lambda X: lc.Constant.int(_int32, X)

def _numpy_array_element(ndarray_ptr, idx, builder):
    ptr_to_element = builder.gep(ndarray_ptr,
                                 map(_make_const_int, [0, _head_len + idx]))
    return builder.load(ptr_to_element)

def _get_pyarray_getptr(module):
    '''
    For reference:

        void *
        PyArray_GetPtr(PyArrayObject *obj, npy_intp* ind)
        {
            int n = PyArray_NDIM(obj);
            npy_intp *strides = PyArray_STRIDES(obj);
            char *dptr = PyArray_DATA(obj);

            while (n--) {
                dptr += (*strides++) * (*ind++);
            }
            return (void *)dptr;
        }
    '''
    function_type = lc.Type.function(_void_star, [_numpy_array, _intp_star])
    function = module.get_or_insert_function(function_type, 'PyArray_GetPtr_inline')
    if function.basic_block_count!=0:
        # Already implemented in the module
        return function

    # set linkage and attributes
    function.add_attribute(lc.ATTR_ALWAYS_INLINE) # force inline
    # function.linkage = lc.LINKAGE_INTERNAL

    print(function)

    # implement the function
    bb_entry = function.append_basic_block('entry')
    bb_while_cond = function.append_basic_block('while.cond')
    bb_while_body = function.append_basic_block('while.body')
    bb_ret = function.append_basic_block('return')

    # initialize
    builder = lc.Builder.new(bb_entry)

    ndarray_element = lambda X: _numpy_array_element(ndarray_ptr, X, builder)

    ndarray_ptr = function.args[0]
    dptr, nd, strides = map(ndarray_element, [0, 1, 3])
    dptr.name = 'dptr'
    nd.name = 'nd'
    strides.name = 'strides'

    builder.branch(bb_while_cond)               # branch to while cond

    # while (n--)
    builder.position_at_end(bb_while_cond)

    nd_phi = builder.phi(nd.type, name='nd_phi')
    strides_phi = builder.phi(strides.type, name='strides_phi')
    ind_phi = builder.phi(function.args[1].type, name='ind_phi')
    dptr_phi = builder.phi(dptr.type, name='dptr_phi')

    nd_phi.add_incoming(nd, bb_entry)

    nd_minus_one = builder.sub(nd_phi, _make_const_int(1), name='nd_minus_one')
    nd_phi.add_incoming(nd_minus_one, bb_while_body)

    pred = builder.icmp(lc.ICMP_NE, nd_phi, _make_const_int(0))

    strides_phi.add_incoming(strides, bb_entry)

    ind_phi.add_incoming(function.args[1], bb_entry)
    
    dptr_phi.add_incoming(dptr, bb_entry)

    builder.cbranch(pred, bb_while_body, bb_ret)

    # dptr += (*strides++) * (*ind++);
    builder.position_at_end(bb_while_body)

    strides_next = builder.gep(strides_phi, [_make_const_int(1)], name='strides_next')
    strides_phi.add_incoming(strides_next, bb_while_body)

    ind_next = builder.gep(ind_phi, [_make_const_int(1)], name='ind_next')
    ind_phi.add_incoming(ind_next, bb_while_body)

    stride_value = builder.load(strides_phi)
    ind_value = builder.load(builder.bitcast(ind_phi, strides_phi.type))
    dptr_next = builder.gep(dptr_phi, [builder.mul(stride_value, ind_value)], name='dptr_next')
    dptr_phi.add_incoming(dptr_next, bb_while_body)

    builder.branch(bb_while_cond)

    # return (void *) dptr;
    builder.position_at_end(bb_ret)

    builder.ret(dptr_phi)

    # check generated code
    function.verify()
            
    return function

# ______________________________________________________________________

# For reference:
#    typedef struct {
#    PyObject_HEAD                   // indices (skipping the head)
#    char *data;                     // 0
#    int nd;                         // 1
#    int *dimensions, *strides;      // 2, 3
#    PyObject *base;                 // 4
#    PyArray_Descr *descr;           // 5
#    int flags;                      // 6
#    } PyArrayObject;

class TestMultiarrayAPI(unittest.TestCase):
    def test_call_PyArray_Zeros(self):
        ma_obj = ma.MultiarrayAPI()
        module = lc.Module.new('test_module')
        ma_obj.set_PyArray_API(module)
        test_fn = module.add_function(lc.Type.function(_numpy_array,
                                                       [_numpy_array]),
                                      'test_fn')
        bb = test_fn.append_basic_block('entry')
        builder = lc.Builder.new(bb)
        pyarray_zeros = ma_obj.load_PyArray_Zeros(module, builder)
        arg = test_fn.args[0]
        largs = [
            builder.load(
                builder.gep(arg,
                            [lc.Constant.int(_int32, 0),
                             lc.Constant.int(_int32, _head_len + ofs)]))
            for ofs in (1, 2, 5)] # nd, dimensions, descr
        largs.append(lc.Constant.int(_int32, 0))
        ret_void_ptr = builder.call(pyarray_zeros, largs)
        builder.ret(builder.bitcast(ret_void_ptr, _numpy_array))

        logging.debug(module)

        module.verify()
        
        ee = le.EngineBuilder.new(module).mattrs('-avx').create()
        test_fn_addr = ee.get_pointer_to_function(test_fn)
        py_test_fn = _pyobj_to_pyobj(test_fn_addr)
        test_arr = np.array([1.,2.,3.])
        result = py_test_fn(test_arr)
        self.assertEqual(result.shape, test_arr.shape)
        self.assertTrue((result == 0.).all())

    def test_call_PyArray_AsCArray(self):
        '''
        A test to check PyArray_AsCArray for accessing the C-array in ndarray.
        This is not the recommended way to access elements in ndarray.
        '''
        ma_obj = ma.MultiarrayAPI()
        module = lc.Module.new('test_module_PyArray_AsCArray')
        ma_obj.set_PyArray_API(module)
        test_fn = module.add_function(lc.Type.function(lc.Type.double(), #_numpy_array,
                                                       [_numpy_array]),
                                      'test_fn')
        bb = test_fn.append_basic_block('entry')
        builder = lc.Builder.new(bb)
        pyarray_ascarray = ma_obj.load_PyArray_AsCArray(module, builder)
        pyarray_ascarray_fnty = pyarray_ascarray.type.pointee

        arg_pyobj = test_fn.args[0]

        # prepare arg 1 PyObject** op
        pyobj_ptr = builder.alloca(arg_pyobj.type)
        builder.store(arg_pyobj, pyobj_ptr)
        arg_pyobj_ptr = builder.bitcast(pyobj_ptr, lc.Type.pointer(_void_star))

        # prepare arg 2 void* ptr

        data_ptr = builder.alloca(lc.Type.pointer(lc.Type.double()))
        arg_data_ptr = builder.bitcast(data_ptr, _void_star)

        # prepare arg 3, 4, 5
        ndarray_element = lambda X: _numpy_array_element(arg_pyobj, X, builder)
        nd, dimensions, descr = map(ndarray_element, [1, 2, 5])

        descr_as_void_ptr = builder.bitcast(descr, _void_star)

        # call
        largs = [arg_pyobj_ptr, arg_data_ptr, dimensions, nd, descr_as_void_ptr]


        status = builder.call(pyarray_ascarray, largs)
        # check errors?
        #    builder.ret(status)


        data_array = builder.load(data_ptr)

        data = []
        for i in xrange(3): # The count is fixed for this simple test.
            elem_ptr = builder.gep(data_array, map(_make_const_int, [i]))
            data.append(builder.load(elem_ptr))

        sum_data = builder.fadd(builder.fadd(data[0], data[1]), data[2])
        builder.ret(sum_data)

        # NOTE: The arg_data_ptr is never freed. This is okay only for test here.

        logging.debug(module)

        test_fn.verify()
        module.verify()

        ee = le.EngineBuilder.new(module).mattrs('-avx').create()
        test_fn_addr = ee.get_pointer_to_function(test_fn)

        c_func_type = ctypes.CFUNCTYPE(ctypes.c_double, ctypes.py_object)
        py_test_fn = c_func_type(test_fn_addr)

        test_arr = np.array([1.234, 2.345, 3.567])
        result = py_test_fn(test_arr)

        self.assertEqual(sum(test_arr), result)

    def test_call_PyArray_GetPtr(self):
        '''
        Using PyArray_GetPtr should be the preferred method to access the
        element. The only thing I am concerning is we will miss optimization
        opportunity since LLVM has no information of PyArray_GetPtr. Perhaps,
        It is better to put the definition inside the LLVM module.
        '''

        ma_obj = ma.MultiarrayAPI()
        module = lc.Module.new('test_module_PyArray_GetPtr')
        ma_obj.set_PyArray_API(module)
        test_fn = module.add_function(lc.Type.function(lc.Type.double(),
                                                       [_numpy_array, _int32]),
                                      'test_fn')
        bb = test_fn.append_basic_block('entry')
        builder = lc.Builder.new(bb)
        pyarray_getptr = ma_obj.load_PyArray_GetPtr(module, builder)
        pyarray_getptr_fnty = pyarray_getptr.type.pointee

        # prepare arg 1 PyObject *
        arg_pyobj = test_fn.args[0]

        npy_intp_ty = pyarray_getptr_fnty.args[1].pointee

        # prepare arg 2 npy_intp *
        arg_index = builder.alloca(npy_intp_ty)
        index_as_npy_intp = builder.sext(test_fn.args[1], npy_intp_ty)
        builder.store(index_as_npy_intp, arg_index)

        # call
        largs = [arg_pyobj, arg_index]
        elemptr = builder.call(pyarray_getptr, largs)

        # return the loaded element at the index specified
        elemptr_as_double_ptr = builder.bitcast(elemptr, lc.Type.pointer(lc.Type.double()))
        builder.ret(builder.load(elemptr_as_double_ptr))

        logging.debug(module)

        test_fn.verify()
        module.verify()

        ee = le.EngineBuilder.new(module).mattrs('-avx').create()
        test_fn_addr = ee.get_pointer_to_function(test_fn)

        c_func_type = ctypes.CFUNCTYPE(ctypes.c_double, ctypes.py_object, ctypes.c_int)
        py_test_fn = c_func_type(test_fn_addr)

        test_arr = np.array([1.234, 2.345, 3.567])

        for idx, val in enumerate(test_arr):
            result = py_test_fn(test_arr, idx)
            self.assertEqual(val, result)

    def test_call_PyArray_GetPtr_inline(self):
        '''
        Let's try implementing PyArray_GetPtr inside LLVM and allow inlining.
        '''
        module = lc.Module.new('test_module_PyArray_GetPtr_inline')


        test_fn = module.add_function(lc.Type.function(lc.Type.double(),
                                                       [_numpy_array, _int32]),
                                      'test_fn')

        bb = test_fn.append_basic_block('entry')
        builder = lc.Builder.new(bb)
        pyarray_getptr = _get_pyarray_getptr(module)
        pyarray_getptr_fnty = pyarray_getptr.type.pointee

        # prepare arg 1 PyObject *
        arg_pyobj = test_fn.args[0]

        npy_intp_ty = pyarray_getptr_fnty.args[1].pointee

        # prepare arg 2 npy_intp *
        arg_index = builder.alloca(npy_intp_ty)
        index_as_npy_intp = builder.sext(test_fn.args[1], npy_intp_ty)
        builder.store(index_as_npy_intp, arg_index)

        # call
        largs = [arg_pyobj, arg_index]
        elemptr = builder.call(pyarray_getptr, largs)

        # return the loaded element at the index specified
        elemptr_as_double_ptr = builder.bitcast(elemptr, lc.Type.pointer(lc.Type.double()))
        builder.ret(builder.load(elemptr_as_double_ptr))

        logging.debug(module)

        test_fn.verify()

        module.verify()

        ee = le.EngineBuilder.new(module).mattrs('-avx').create()
        test_fn_addr = ee.get_pointer_to_function(test_fn)

        c_func_type = ctypes.CFUNCTYPE(ctypes.c_double, ctypes.py_object, ctypes.c_int)
        py_test_fn = c_func_type(test_fn_addr)

        test_arr = np.array([1.234, 2.345, 3.567])

        for idx, val in enumerate(test_arr):
            result = py_test_fn(test_arr, idx)
            self.assertEqual(val, result)


# ______________________________________________________________________

if __name__ == "__main__":
    unittest.main()

# ______________________________________________________________________
# End of test_multiarray_api.py



"""
>>> get_list('world')
[1, 'hello', 2.0, 'world']
>>> get_tuple('world')
(1L, 'hello', 2.0, 'world')
>>> get_dict('world') == {"hello": 1, 2.0: 'world'}
True
"""

import sys
from numba import *

myglobal = 20

@autojit(backend='ast')
def get_list(x):
    return [1, "hello", 2.0, x]

@autojit(backend='ast')
def get_tuple(x):
    return (1, "hello", 2.0, x)

@autojit(backend='ast')
def get_dict(x):
    return {"hello": 1, 2.0: x}

if __name__ == '__main__':
    import numba
    from numba.testing.test_support import rewrite_doc
    __doc__ = rewrite_doc(__doc__)
    numba.testing.testmod()



"Simple utilities related to extension types"

#------------------------------------------------------------------------
# Type checking
#------------------------------------------------------------------------

def is_numba_class(py_class):
    return (hasattr(py_class, '__numba_ext_type') or
            is_autojit_class(py_class))

def is_autojit_class(py_class):
    "Returns whether the given class is an unspecialized autojit class"
    return hasattr(py_class, "__is_numba_autojit")

def get_all_numba_bases(py_class):
    seen = set()

    bases = []
    for base in py_class.__mro__[::-1]:
        if is_numba_class(base) and base.exttype not in seen:
            seen.add(base.exttype)
            bases.append(base)

    return bases[::-1]

def get_numba_bases(py_class):
    return list(filter(is_numba_class, py_class.__bases__))

def get_class_dict(unspecialized_autojit_py_class):
    return unspecialized_autojit_py_class.__numba_class_dict


import unittest
from numba import jit

def loop_all(begin, end, mask):
    hash = 0
    for i in xrange(begin, end):
        hash ^= i | ((hash << 1) & mask)
    return hash

def loop_all_simpler(begin, end):
    hash = 0
    for i in xrange(begin, end):
        hash ^= begin + hash
    return hash

class TestBitwiseLoop(unittest.TestCase):
    def test_loop_all_simpler(self):
        fn = jit('uint32(uint32, uint32)')(loop_all_simpler)
        msg = "a=%s b=%s got=%s exp=%s"
        a, b = 0, 2**16 - 1
        exp = fn.py_func(a, b)
        got = fn(a, b)
        self.assertTrue(exp == got, msg % (a, b, got, exp))

    def test_loop_all(self):
        fn = jit('uint32(uint32, uint32, uint32)')(loop_all)
        msg = "a=%s b=%s got=%s exp=%s"
        a, b = 0, 2**16 - 1
        c = 0xffffffff
        exp = fn.py_func(a, b, c)
        got = fn(a, b, c)
        self.assertTrue(exp == got, msg % (a, b, got, exp))

if __name__ == '__main__':
    # TestBitwiseLoop('test_loop_all_simpler').debug()
    unittest.main()


# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import traceback

from numba.minivect.minierror import Error

__all__ = ["Error", "NumbaError", "InternalError", "InvalidTemplateError"]

def format_pos(node):
    if node is not None and hasattr(node, 'lineno'):
        return format_postup((node.lineno, node.col_offset))
    else:
        return ""

def format_postup(tup):
    lineno, col_offset = tup
    return "%s:%s: " % (lineno - 1, col_offset)

class NumbaError(Error):
    "Some error happened during compilation"

    def __init__(self, node, msg=None, *args):
        if msg is None:
            node, msg = None, node

        self.node = node
        self.msg = msg
        self.args = args

    def __str__(self):
        try:
            pos = format_pos(self.node)
            msg = "%s%s %s" % (pos, self.msg, " ".join(map(str, self.args)))
            return msg.rstrip()
        except:
            traceback.print_exc()
            return "<internal error creating numba error message>"


class InternalError(Error):
    "Indicates a compiler bug"

class _UnknownAttribute(Error):
    pass

class InvalidTemplateError(Error):
    "Raised for invalid template type specifications"



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import linregr_python, linregr_numba, linregr_numbapro
import numpy as np
import pylab
from timeit import default_timer as timer

def populate_data(N):
    noise = np.random.random(N).astype(np.float64)
    X = np.arange(N, dtype=np.float64)
    slope = 3
    Y = noise * (slope * X)
    return X, Y

def run(gradient_descent, X, Y, iterations=1000, alpha=1e-6):
    theta = np.empty(2, dtype=X.dtype)

    ts = timer()
    gradient_descent(X, Y, theta, alpha, iterations)
    te = timer()

    timing = te - ts

    print("x-offset = {}    slope = {}".format(*theta))
    print("time elapsed: {} s".format(timing))

    return theta, timing


def plot(X, theta, c='r'):
    result = theta[0] + theta[1] * X
    pylab.plot(X, result, c=c, linewidth=2)


def main():
    N = 50
    X, Y = populate_data(N)
    pylab.scatter(X, Y, marker='o', c='b')
    pylab.title('Linear Regression')

    print('Python'.center(80, '-'))
    theta_python, time_python = run(linregr_python.gradient_descent, X, Y)

    print('Numba'.center(80, '-'))
    theta_numba, time_numba  = run(linregr_numba.gradient_descent, X, Y)

    print('NumbaPro'.center(80, '-'))
    theta_numbapro, time_numbapro  = run(linregr_numbapro.gradient_descent, X, Y)

    # make sure all method yields the same result
    assert np.allclose(theta_python, theta_numba)
    assert np.allclose(theta_python, theta_numbapro)

    print('Summary'.center(80, '='))
    print('Numba speedup %.1fx' % (time_python / time_numba))
    print('NumbaPro speedup %.1fx' % (time_python / time_numbapro))

    plot(X, theta_numba, c='r')
    pylab.show()

if __name__ == '__main__':
    main()




from numba import *
from numba.testing.test_support import parametrize, main

from numba.exttypes.tests import test_extension_methods

Base1 = test_extension_methods.make_base(autojit)

@autojit
class Base2(object):

    def __init__(self, myfloat):
        self.value = myfloat

    def getvalue(self):
        return self.value

    # @staticmethod
    # def static1():
    #     return 10.0
    #
    # @staticmethod
    # def static2(value):
    #     return value * 2
    #
    # @staticmethod
    # def static3(value1, value2):
    #     return value1 * value2
    #
    # @staticmethod
    # @double(double, double)
    # def static4(value1, value2):
    #     return value1 * value2
    #
    # @classmethod
    # @double()
    # def class1(cls):
    #     return 10.0
    #
    # @double(double)
    # @classmethod
    # def class2(cls, value):
    #     return value * 2
    #
    # @double(double, double)
    # @classmethod
    # def class3(cls, value1, value2):
    #     return value1 * value2
    #
    # @classmethod
    # @double(double, double)
    # def class4(cls, value1, value2):
    #     return value1 * value2

#------------------------------------------------------------------------
# Tests
#------------------------------------------------------------------------

obj1 = Base1(10.0)
specialized_class1 = Base1[{'value': double}]
obj2 = Base1(11)

@parametrize(Base1, specialized_class1, obj1, obj2)
def test_staticmethods(obj):
    assert obj.static1() == 10.0
    assert obj.static2(10.0) == 20.0
    assert obj.static3(5.0, 6.0) == 30.0
    assert obj.static4(5.0, 6.0) == 30.0

@parametrize(Base1, specialized_class1, obj1, obj2)
def test_classmethods(obj):
    assert obj.class1() == 10.0
    assert obj.class2(10.0) == 20.0
    assert obj.class3(5.0, 6.0) == 30.0
    assert obj.class4(5.0, 6.0) == 30.0

@parametrize(obj1)
def test_specialized_unbound(obj):
    assert type(obj) is specialized_class1
    assert specialized_class1.getvalue(obj) == 10.0

@parametrize(obj2)
def test_specialized_unbound2(obj):
    assert issubclass(type(obj), Base1)
    assert type(obj).getvalue(obj) == 11


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.nodes import *
from numba.exttypes.types import methods

class ExtTypeAttribute(ExprNode):

    _fields = ['value']

    def __init__(self, value, attr, variable, ctx, ext_type, **kwargs):
        super(ExtTypeAttribute, self).__init__(**kwargs)
        self.value = value
        self.attr = attr
        self.type = variable.type
        self.variable = variable
        self.ctx = ctx
        self.ext_type = ext_type

    def __repr__(self):
        return "%s.%s" % (self.value, self.attr)

    @classmethod
    def from_known_attribute(cls, value, attr, ctx, ext_type):
        """
        Create an extension type attribute node if the attribute is known
        to exist (and isn't being inferred)
        """
        assert attr in ext_type.attributedict

        import numba.symtab

        variable = numba.symtab.Variable(ext_type.attributedict[attr])
        return cls(value, attr, variable, ctx, ext_type)

class NewExtObjectNode(ExprNode):
    """
    Instantiate an extension type. Currently unused.
    """

    _fields = ['args']

    def __init__(self, ext_type, args, **kwargs):
        super(NewExtObjectNode, self).__init__(**kwargs)
        self.ext_type = ext_type
        self.args = args

class ExtensionMethod(ExprNode):

    _fields = ['value']
    call_node = None

    def __init__(self, obj, attr, method, **kwargs):
        super(ExtensionMethod, self).__init__(**kwargs)
        ext_type = obj.variable.type
        assert ext_type.is_extension
        self.value = obj
        self.attr = attr

        self.ext_type = ext_type

        self.initialize_type(method)

    def initialize_type(self, method):
        self.type = method.signature

    def __repr__(self):
        return "%s.%s" % (self.value, self.attr)

class AutojitExtensionMethod(ExtensionMethod):

    def initialize_type(self, method):
        self.type = methods.AutojitMethodType()


#class ExtensionMethodCall(Node):
#    """
#    Low level call that has resolved the virtual method.
#    """
#
#    _fields = ['vmethod', 'args']
#
#    def __init__(self, vmethod, self_obj, args, signature, **kwargs):
#        super(ExtensionMethodCall, self).__init__(**kwargs)
#        self.vmethod = vmethod
#        self.args = args
#        self.signature = signature
#        self.type = signature



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import ast
from itertools import chain

from numba import nodes
from .debug import logger

#------------------------------------------------------------------------
# Kill unused Phis
#------------------------------------------------------------------------

def kill_unused_phis(cfg):
    changed = True
    while changed:
        changed = _kill_unused_phis(cfg)

def kill_phi(block, phi):
    logger.debug("Killing phi: %s", phi)

    block.symtab.pop(phi.variable.renamed_name)

    for incoming_var in phi.incoming:
        # A single definition can reach a block multiple times,
        # remove all references
        refs = [ref for ref in incoming_var.cf_references
                        if ref.variable is not phi.variable]
        incoming_var.cf_references = refs

def kill_unused_phis(cfg):
    """
    Used before running type inference.

    Kill phis which are not referenced. We need to do this bottom-up,
    i.e. in reverse topological dominator-tree order, since in SSA
    a definition always lexically precedes a reference.

    This is important, since it kills any unnecessary promotions (e.g.
    ones to object, which LLVM wouldn't be able to optimize out).

    TODO: Kill phi cycles, or do reachability analysis before inserting phis.
    """
    changed = False

    for block in cfg.blocks[::-1]:
        phi_nodes = []

        for i, phi in enumerate(block.phi_nodes):
            if phi.variable.cf_references:
                # Used phi
                # logger.info("Used phi %s, %s" % (phi, phi.variable.cf_references))
                phi_nodes.append(phi)
            else:
                # Unused phi
                changed = True

                kill_phi(block, phi)

        block.phi_nodes = phi_nodes

    return changed

#------------------------------------------------------------------------
# Iterate over all phi nodes or variables
#------------------------------------------------------------------------

def iter_phis(flow):
    "Iterate over all phi nodes"
    return chain(*[block.phi_nodes for block in flow.blocks])

def iter_phi_vars(flow):
    "Iterate over all phi nodes"
    for phi_node in iter_phis(flow):
        yield phi_node.variable

#------------------------------------------------------------------------
# Specialization code for SSA
#------------------------------------------------------------------------

def specialize_ssa(funcdef):
    """
    Handle phi nodes:

        1) Handle incoming variables which are not initialized. Set
           incoming_variable.uninitialized_value to a constant 'bad'
           value (e.g. 0xbad for integers, NaN for floats, NULL for
           objects)

        2) Handle incoming variables which need promotions. An incoming
           variable needs a promotion if it has a different type than
           the the phi. The promotion happens in each ancestor block that
           defines the variable which reaches us.

           Promotions are set separately in the symbol table, since the
           ancestor may not be our immediate parent, we cannot introduce
           a rename and look up the latest version since there may be
           multiple different promotions. So during codegen, we first
           check whether incoming_type == phi_type, and otherwise we
           look up the promotion in the parent block or an ancestor.
    """
    for phi_node in iter_phis(funcdef.flow):
        specialize_phi(phi_node)

def specialize_phi(node):
    for parent_block, incoming_var in node.find_incoming():
        if incoming_var.type.is_uninitialized:
            incoming_type = incoming_var.type.base_type or node.type
            bad = nodes.badval(incoming_type)
            incoming_var.type.base_type = incoming_type
            incoming_var.uninitialized_value = bad
            # print incoming_var

        elif not incoming_var.type == node.type:
            # Create promotions for variables with phi nodes in successor
            # blocks.
            incoming_symtab = incoming_var.block.symtab
            if (incoming_var, node.type) not in node.block.promotions:
                # Make sure we only coerce once for each destination type and
                # each variable
                incoming_var.block.promotions.add((incoming_var, node.type))

                # Create promotion node
                name_node = nodes.Name(id=incoming_var.renamed_name,
                                       ctx=ast.Load())
                name_node.variable = incoming_var
                name_node.type = incoming_var.type
                coercion = name_node.coerce(node.type)
                promotion = nodes.PromotionNode(node=coercion)

                # Add promotion node to block body
                incoming_var.block.body.append(promotion)
                promotion.variable.block = incoming_var.block

                # Update symtab
                incoming_symtab.promotions[incoming_var.name,
                                           node.type] = promotion
            else:
                promotion = incoming_symtab.lookup_promotion(
                    incoming_var.name, node.type)

    return node

#------------------------------------------------------------------------
# Handle phis during code generation
#------------------------------------------------------------------------

def process_incoming(phi_node):
    """
    Add all incoming phis to the phi instruction.

    Handle promotions by using the promoted value from the incoming block.
    E.g.

        bb0: if C:
        bb1:     x = 2
             else:
        bb2:     x = 2.0

        bb3: x = phi(x_bb1, x_bb2)

    has a promotion for 'x' in basic block 1 (from int to float).
    """
    var = phi_node.variable
    phi = var.lvalue

    for parent_block, incoming_var in phi_node.find_incoming():
        if incoming_var.type.is_uninitialized:
            pass
        elif not incoming_var.type == phi_node.type:
            promotion = parent_block.symtab.lookup_promotion(var.name,
                                                             phi_node.type)
            incoming_var = promotion.variable

        assert incoming_var.lvalue, incoming_var
        assert parent_block.exit_block, parent_block

        phi.add_incoming(incoming_var.lvalue,
                         parent_block.exit_block)

        if phi_node.type.is_array:
            nodes.update_preloaded_phi(phi_node.variable,
                                       incoming_var,
                                       parent_block.exit_block)


def handle_phis(flow):
    """
    Update all our phi nodes after translation is done and all Variables
    have their llvm values set.
    """
    if flow is None:
        return

    for phi_node in iter_phis(flow):
        process_incoming(phi_node)



# issue: #33
# Thanks to Stefan van der Walt

import numpy as np

from numba.decorators import jit
from numba import float64, int32

@jit(argtypes=[float64[:, :], float64[:, :], float64[:, :]])
def ndot(A, B, out):
    rows_A, cols_A = A.shape
    rows_B, cols_B = B.shape

    # Take each row in A
    for i in range(rows_A):

        # And multiply by every column in B
        for j in range(cols_B):
            s = 0.0
            for k in range(cols_A):
                s = s + A[i, k] * B[k, j]

            out[i, j] = s

    return out

def test_dot():
    A = np.random.random((10, 10))
    B = np.random.random((10, 10))
    C = np.empty_like(A)

    assert np.allclose(np.dot(A, B), ndot(A, B, C))

if __name__ == '__main__':
    test_dot()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import


from numba import *
import numpy as np

@jit(void(f8[:]))
def ff(T):
    for j in range(100): #reduce 100 to 10 get no error
        T[j]=1.0

x=np.ones(100,dtype=np.double)
ff(x)
assert np.all(x == 1.0)



"""
>>> int() == 0
True
>>> convert_int(2.5) == 2
True
>>> convert_to_int('FF', 16) == 255
True
"""

import sys

from numba import *

@autojit(backend='ast')
def empty_int():
    x = float()
    return x

@autojit(backend='ast')
def convert_int(x):
    return int(x)

@autojit(backend='ast')
def convert_to_int(s, base):
    return int(s, base)

if __name__ == '__main__':
    import numba
    numba.testing.testmod()



from numba import *

jitv = jit(void(), warnstyle='simple')
jitvi = jit(void(int_), warnstyle='simple')
jitvii = jit(void(int_, int_), warnstyle='simple')
jitii = jit(int_(int_), warnstyle='simple')
jitiii = jit(int_(int_, int_), warnstyle='simple')


def simple():
    """
    >>> jitv(simple)
    Traceback (most recent call last):
        ...
    NumbaError: 17:10: local variable 'a' referenced before assignment
    """
    print(a)
    a = 0

def simple2(arg):
    """
    >>> result = jitii(simple2)
    Warning 27:11: local variable 'a' might be referenced before assignment
    """
    if arg > 0:
        a = 1
    return a

def simple_pos(arg):
    """
    >>> result = jitii(simple_pos)
    """
    if arg > 0:
        a = 1
    else:
        a = 0
    return a

def ifelif(c1, c2):
    """
    >>> result = jitiii(ifelif)
    Warning 51:11: local variable 'a' might be referenced before assignment
    """
    if c1 == 1:
        if c2:
            a = 1
        else:
            a = 2
    elif c1 == 2:
        a = 3
    return a

def nowimpossible(a):
    """
    >>> result = jitvi(nowimpossible)
    Warning 61:14: local variable 'b' might be referenced before assignment
    """
    if a:
        b = 1
    if a:
        print(b)

def fromclosure():
    """
    >> result = jitv(fromclosure)
    """
    def bar():
        print(a)
    a = 1
    return bar

# Should work ok in both py2 and py3
def list_comp(a):
    return [i for i in a]

def set_comp(a):
    return set(i for i in a)

#def dict_comp(a):
#    return {i: j for i, j in a}

# args and kwargs
def generic_args_call(*args, **kwargs):
    return args, kwargs

def cascaded(x):
    print((a, b))
    a = b = x

def from_import():
    print(bar)
    from foo import bar

def regular_import():
    print(foo)
    import foo

def raise_stat():
    try:
        raise exc(msg)
    except:
        pass
    exc = ValueError
    msg = 'dummy'

def defnode_decorator():
    @decorator
    def foo():
        pass
    def decorator():
        pass

def defnode_default():
    def foo(arg=default()):
        pass
    def default():
        pass

def class_bases():
    class foo(bar):
        pass
    class bar(object):
        pass

def class_decorators():
    @decorator
    class foo(object):
        pass
    def decorator(cls):
        return cls

def uninitialized_augmented_assignment():
    """
    >>> func = jitv(uninitialized_augmented_assignment)
    Traceback (most recent call last):
        ...
    NumbaError: 139:4: local variable 'x' referenced before assignment
    """
    x += 1


def uninitialized_augmented_assignment_loop():
    """
    >>> func = jitv(uninitialized_augmented_assignment_loop)
    Warning 148:8: local variable 'x' might be referenced before assignment
    """
    for i in range(10):
        x += 1

    x = 0

if __name__ == "__main__":
    import numba
    numba.testing.testmod()



# -*- coding: utf-8 -*-
"""
Type inference for NumPy binary ufuncs and their methods.
"""
from __future__ import print_function, division, absolute_import

import numpy as np

from numba import *
from numba.minivect import minitypes
from numba import typesystem
from numba.type_inference.module_type_inference import (module_registry,
                                                        register,
                                                        register_inferer,
                                                        register_unbound)
from numba.typesystem import get_type
from numba.type_inference.modules.numpymodule import (get_dtype,
                                                      array_from_type,
                                                      promote,
                                                      promote_to_array,
                                                      demote_to_scalar)

#----------------------------------------------------------------------------
# Utilities
#----------------------------------------------------------------------------

def array_of_dtype(a, dtype, static_dtype, out):
    if out is not None:
        return out

    a = array_from_type(a)
    if not a.is_object:
        dtype = _dtype(a, dtype, static_dtype)
        if dtype is not None:
            return a.copy(dtype=dtype)

def _dtype(a, dtype, static_dtype):
    if static_dtype:
        return static_dtype
    elif dtype:
        return dtype.dtype
    elif a.is_array:
        return a.dtype
    elif not a.is_object:
        return a
    else:
        return None

#------------------------------------------------------------------------
# Ufunc Type Strings
#------------------------------------------------------------------------

def numba_type_from_sig(ufunc_signature):
    """
    Convert ufunc type signature string (e.g. 'dd->d') to a FunctionType
    """
    args, ret = ufunc_signature.split('->')
    to_numba = lambda c: minitypes.map_dtype(np.dtype(c))

    signature = to_numba(ret)(*map(to_numba, args))
    return signature

def find_signature(args, signatures):
    for signature in signatures:
        if signature.args == args:
            return signature

def find_ufunc_signature(context, argtypes, signatures):
    """
    Map (float_, double) and [double(double, double),
                              int_(int_, int_),
                              ...]
    to double(double, double)
    """
    signature = find_signature(tuple(argtypes), signatures)

    if signature is not None:
        return signature

    argtype = reduce(context.promote_types, argtypes)
    if not argtype.is_object:
        args = (argtype,) * len(argtypes)
        return find_signature(args, signatures)

    return None

class UfuncTypeInferer(object):
    "Infer types for arbitrary ufunc"

    def __init__(self, ufunc):
        self.ufunc = ufunc
        self.signatures = set(map(numba_type_from_sig, ufunc.types))

    def infer(self, context, argtypes):
        signature = find_ufunc_signature(context, argtypes, self.signatures)
        if signature is None:
            return None
        else:
            return signature.return_type

def register_arbitrary_ufunc(ufunc):
    "Type inference for arbitrary ufuncs"
    ufunc_infer = UfuncTypeInferer(ufunc)

    def infer(context, *args, **kwargs):
        if len(args) != ufunc.nin:
            return object_

        # Find the right ufunc signature
        argtypes = [type.dtype if type.is_array else type for type in args]
        result_type = ufunc_infer.infer(context, argtypes)

        if result_type is None:
            return object_

        # Determine output ndim
        ndim = 0
        for argtype in args:
            if argtype.is_array:
                ndim = max(argtype.ndim, ndim)

        return typesystem.array(result_type, ndim)

    module_registry.register_value(ufunc, infer)
    # module_registry.register_unbound_dotted_value

#----------------------------------------------------------------------------
# Ufunc type inference
#----------------------------------------------------------------------------

def binary_map(context, a, b, out):
    if out is not None:
        return out

    return promote(context, a, b)

def binary_map_bool(context, a, b, out):
    type = binary_map(context, a, b, out)
    if type.is_array:
        return type.copy(dtype=bool_)
    else:
        return bool_

def reduce_(a, axis, dtype, out, static_dtype=None):
    if out is not None:
        return out

    dtype_type = _dtype(a, dtype, static_dtype)

    if axis is None:
        # Return the scalar type
        return dtype_type

    if dtype_type:
        # Handle the axis parameter
        if axis.is_tuple and axis.is_sized:
            # axis=(tuple with a constant size)
            return typesystem.array(dtype_type, a.ndim - axis.size)
        elif axis.is_int:
            # axis=1
            return typesystem.array(dtype_type, a.ndim - 1)
        else:
            # axis=(something unknown)
            return object_

def reduce_bool(a, axis, dtype, out):
    return reduce_(a, axis, dtype, out, bool_)

def accumulate(a, axis, dtype, out, static_dtype=None):
    return demote_to_scalar(array_of_dtype(a, dtype, static_dtype, out))

def accumulate_bool(a, axis, dtype, out):
    return accumulate(a, axis, dtype, out, bool_)

def reduceat(a, indices, axis, dtype, out, static_dtype=None):
    return accumulate(a, axis, dtype, out, static_dtype)

def reduceat_bool(a, indices, axis, dtype, out):
    return reduceat(a, indices, axis, dtype, out, bool_)

def outer(context, a, b, static_dtype=None):
    a = array_of_dtype(a, None, static_dtype, out=None)
    if a and a.is_array:
        return a.dtype[:, :]

def outer_bool(context, a, b):
    return outer(context, a, b, bool_)

#------------------------------------------------------------------------
# Binary Ufuncs
#------------------------------------------------------------------------

binary_ufuncs_compare = (
    # Comparisons
    'greater',
    'greater_equal',
    'less',
    'less_equal',
    'not_equal',
    'equal',
)

binary_ufuncs_logical = (
    # Logical ufuncs
    'logical_and',
    'logical_or',
    'logical_xor',
    'logical_not',
)

binary_ufuncs_bitwise = (
    # Bitwise ufuncs
    'bitwise_and',
    'bitwise_or',
    'bitwise_xor',
    'left_shift',
    'right_shift',
)

binary_ufuncs_arithmetic = (
    # Arithmetic ufuncs
    'add',
    'subtract',
    'multiply',
    'true_divide',
    'floor_divide',
)

if not PY3:
    binary_ufuncs_arithmetic = binary_ufuncs_arithmetic + ('divide', )

#------------------------------------------------------------------------
# Register our type functions
#------------------------------------------------------------------------

register_inferer(np, 'sum', reduce_)
register_inferer(np, 'prod', reduce_)

def register_arithmetic_ufunc(register_inferer, register_unbound, binary_ufunc):
    register_inferer(np, binary_ufunc, binary_map)
    register_unbound(np, binary_ufunc, "reduce", reduce_)
    register_unbound(np, binary_ufunc, "accumulate", accumulate)
    register_unbound(np, binary_ufunc, "reduceat", reduceat)
    register_unbound(np, binary_ufunc, "outer", outer)

def register_bool_ufunc(register_inferer, register_unbound, binary_ufunc):
    register_inferer(np, binary_ufunc, binary_map_bool)
    register_unbound(np, binary_ufunc, "reduce", reduce_bool)
    register_unbound(np, binary_ufunc, "accumulate", accumulate_bool)
    register_unbound(np, binary_ufunc, "reduceat", reduceat_bool)
    register_unbound(np, binary_ufunc, "outer", outer_bool)

for binary_ufunc in binary_ufuncs_bitwise + binary_ufuncs_arithmetic:
    register_arithmetic_ufunc(register_inferer, register_unbound, binary_ufunc)

for binary_ufunc in binary_ufuncs_compare + binary_ufuncs_logical:
    register_bool_ufunc(register_inferer, register_unbound, binary_ufunc)



import unittest
import numpy as np
from itertools import product
from numba import jit

def add(a, b):
    return a + b

def sub(a, b):
    return a - b

def mul(a, b):
    return a * b

def div(a, b):
    return a // b

def boundary_range(s=0):
    maxint = 2**32 - 1
    blimit = 10
    for i in xrange(s, blimit):
        yield i
    i = blimit
    while i < maxint:
        i &= maxint
        yield i
        i = i << 1
    for i in xrange(maxint - blimit, maxint):
        yield i

def boundary_range_signed(s=0):
    maxint = 2**16 - 1
    blimit = 10
    for sign in (1, -1):
        for i in xrange(s, blimit):
            yield sign * i
        i = blimit
        while i < maxint:
            yield sign * i
            i = i << 1
        for i in xrange(maxint - blimit, maxint):
            yield sign * i


class TestBinaryOps(unittest.TestCase):
    def template(self, pyfn, op, s=0):
        msg = "%s %s %s -> %s (expect %s)"
        types = 'uint32', 'float64'
        for ty in types:
            signature = '%s(%s,%s)' % (ty, ty, ty)
            fn = jit(signature)(pyfn)
            for a, b in product(boundary_range(s=s), boundary_range(s=s)):
                if ty in ['float64']:
                    a, b = float(a), float(b)

                exp = pyfn(a, b)
                if ty in ['uint32']:
                    exp &= 0xffffffff
                try:
                    got = fn(a, b)
                except:
                    print('Exception raised at fn=%s a=%s b=%s, exp=%s ty=%s' %
                          (pyfn, a, b, exp, ty))
                    raise
                self.assertTrue(exp == got, msg % (a, op, b, got, exp))

    def template2(self, pyfn, op, s=0, w=True):
        msg = "%s %s %s -> %s (expect %s)"
        types = 'int32', 'float64'
        for ty in types:
            signature = '%s(%s,%s)' % (ty, ty, ty)
            fn = jit(signature)(pyfn)
            for a, b in product(boundary_range_signed(s=s),
                                boundary_range_signed(s=s)):
                if ty in ['float64']:
                    a, b = float(a), float(b)

                if w:
                    exp = pyfn(np.asarray(a, dtype=ty), np.asarray(b, dtype=ty))
                else:
                    exp = pyfn(a, b)
                try:
                    got = fn(a, b)
                except:
                    print('Exception raised at fn=%s a=%s b=%s, exp=%s ty=%s' %
                          (pyfn, a, b, exp, ty))
                    raise
                self.assertTrue(exp == got, msg % (a, op, b, got, exp))


    def test_add(self):
        self.template(add, '+')
        self.template2(add, '+')

    def test_sub(self):
        self.template(sub, '-')
        self.template2(sub, '-')

    def test_mul(self):
        self.template(mul, '*')
        self.template2(mul, '*')

    def test_div(self):
        self.template(div, '//', s=1)
        self.template2(div, '//', s=1, w=False)

if __name__ == '__main__':
    unittest.main()



import numpy as np

from numba.testing.test_support import *
from numba.minivect import minitypes
from numba import pipeline, environment, functions, error

def construct_infer_pipeline():
    order = environment.default_pipeline_order
    dump_cfg_index = order.index('dump_cfg')
    return pipeline.ComposedPipelineStage(order[:dump_cfg_index+1])

def functype(restype=None, argtypes=()):
    return minitypes.FunctionType(return_type=restype, args=list(argtypes))

def lookup(block, var_name):
    var = None
    try:
        var = block.symtab.lookup_most_recent(var_name)
    except (AssertionError, KeyError):
        if block.idom:
            var = lookup(block.idom, var_name)

    return var

def types(symtab, *varnames):
    return tuple(symtab[varname].type for varname in varnames)

def infer(func, signature=functype(), warn=True, **kwargs):
    func_ast = functions._get_ast(func)
    env = environment.NumbaEnvironment.get_environment(kwargs.get('env', None))
    infer_pipe = env.get_or_add_pipeline('infer', construct_infer_pipeline)
    kwargs.update(warn=warn, pipeline_name='infer')
    pipe, (signature, symtab, func_ast) = pipeline.run_pipeline2(
        env, func, func_ast, signature, **kwargs)
    last_block = func_ast.flow.blocks[-2]
    symbols = {}
    #for block in ast.flow.blocks: print block.symtab
    for var_name, var in symtab.iteritems():
        if not var.parent_var and not var.is_constant:
            var = lookup(last_block, var_name)
            if var:
                symbols[var_name] = var

    return signature, symbols

class Value(object):
    def __init__(self, value):
        self.value = value

    def __repr__(self):
        return "Value(%s)" % self.value

    def __int__(self):
        return self.value

values = [Value(i) for i in range(10)]

@autojit
def test_reassign(obj):
    """
    >>> test_reassign(object())
    'hello'
    >>> sig, syms = infer(test_reassign.py_func, functype(None, [object_]))
    >>> sig
    const char * (*)(object_)
    >>> syms['obj'].type
    const char *
    """
    obj = 1
    obj = 1.0
    obj = 1 + 4j
    obj = 2
    obj = "hello"
    return obj

@autojit
def test_if_reassign(obj1, obj2):
    """
    >>> test_if_reassign(*values[:2])
    (4.0, 5.0)
    >>> sig, syms = infer(test_if_reassign.py_func,
    ...                   functype(None, [object_] * 2))
    >>> types(syms, 'obj1', 'obj2')
    (double, object_)
    """
    x = 4.0
    y = 5.0
    z = 6.0
    if int(obj1) < int(obj2):
        obj1 = x
        obj2 = y
    else:
        obj1 = z

    return obj1, obj2

@autojit
def test_if_reassign2(value, obj1, obj2):
    """
    >>> test_if_reassign2(0, *values[:2])
    (4.0, 5.0, 'egel')
    >>> test_if_reassign2(1, *values[:2])
    ('hello', 'world', 'hedgehog')
    >>> test_if_reassign2(2, *values[:2])
    ([Value(0)], Value(12), 'igel')

    >>> sig, syms = infer(test_if_reassign2.py_func,
    ...                   functype(None, [int_, object_, object_]))
    >>> types(syms, 'obj1', 'obj2', 'obj3')
    (object_, object_, const char *)
    """
    x = 4.0
    y = 5.0
    z = "hedgehog"
    if value < 1:
        obj1 = x
        obj2 = y
        obj3 = "egel"
    elif value < 2:
        obj1 = "hello"
        obj2 = "world"
        obj3 = z
    else:
        obj1 = [obj1]
        obj2 = Value(12)
        obj3 = "igel"

    return obj1, obj2, obj3

@autojit_py3doc
def test_for_reassign(obj1, obj2, obj3, obj4):
    """
    >>> test_for_reassign(*values[:4])
    (9L, Value(1), 2L, 5L)
    >>> sig, syms = infer(test_for_reassign.py_func,
    ...                   functype(None, [object_] * 4))
    >>> types(syms, 'obj1', 'obj2', 'obj3', 'obj4')
    (object_, object_, int, Py_ssize_t)
    """
    for i in range(10):
        obj1 = i

    for i in range(0):
        obj2 = i

    for i in range(10):
        obj3 = i
    else:
        obj3 = 2 # This definition kills any previous definition

    for i in range(5, 10):
        obj4 = i
        break
    else:
        obj4 = 0

    return obj1, obj2, obj3, obj4

@autojit_py3doc
def test_while_reassign(obj1, obj2, obj3, obj4):
    """
    >>> test_while_reassign(*values[:4])
    (9L, Value(1), 2L, 5L)
    >>> sig, syms = infer(test_while_reassign.py_func,
    ...                   functype(None, [object_] * 4))
    >>> types(syms, 'obj1', 'obj2', 'obj3', 'obj4')
    (object_, object_, int, int)
    """
    i = 0
    while i < 10:
        obj1 = i
        i += 1

    i = 0
    while i < 0:
        obj2 = i
        i += 1

    i = 0
    while i < 10:
        obj3 = i
        i += 1
    else:
        obj3 = 2 # This definition kills any previous definition

    i = 5
    while i < 10:
        obj4 = i
        i += 1
        break
    else:
        obj4 = 0

    return obj1, obj2, obj3, obj4

@autojit(warn=False)
def test_conditional_assignment(value):
    """
    >>> test_conditional_assignment(0)
    array([ 1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.], dtype=float32)

    >>> test_conditional_assignment(1)
    Traceback (most recent call last):
        ...
    UnboundLocalError: 210:11: obj1
    """
    if value < 1:
        obj1 = np.ones(10, dtype=np.float32)

    return obj1


#
### Test for errors
#
@autojit
def test_error_array_variable1(value, obj1):
    """
    >>> test_error_array_variable1(0, object())
    Traceback (most recent call last):
        ...
    TypeError: Arrays must have consistent types in assignment for variable 'obj1': 'float32[:]' and 'object_'
    """
    if value < 1:
        obj1 = np.empty(10, dtype=np.float32)
    
    return obj1

def test():
    from . import test_cfg_type_infer
    testmod(test_cfg_type_infer)

if __name__ == '__main__':
    testmod()
#else:
#    test()



from numba import *

a = 10
b = 11
c = 12

def jitter():
    a = 20
    b = 21
    c = 22

    @jit(object_())
    def func():
        return a, c

    return func

func = jitter()
assert func() == (20, 22)



import doctest

import numba

@numba.autojit
def func(value):
    """
    >>> func(10.0)
    10.0
    """
    return value

numba.testmod()

@numba.autojit
def func2(value):
    """
    >>> raise ValueError("I am a message")
    Traceback (most recent call last):
        ...
    ValueError: I am a ...
    """

numba.testmod(verbosity=2, optionflags=doctest.ELLIPSIS)



# -*- coding: utf-8 -*-

"""
Test properties of specialized classes and test indexing.
"""

from __future__ import print_function, division, absolute_import

from numba import *

@autojit
class C(object):
    def __init__(self, value):
        self.value = value

obj = C(10.0)
print(type(obj).exttype)

specialized_cls = C[{'value': double}]
print(specialized_cls, C, specialized_cls is C)

assert issubclass(specialized_cls, C)
assert isinstance(obj, C)

try:
    C[{'value': int_}]
except KeyError as e:
    assert e.args[0] == {'value': int_}
else:
    raise Exception



"""
See also numba.tests.test_overflow.
"""

import ctypes
import unittest

import numpy as np
from numba.minivect import minitypes
from numba import *

@autojit(backend='ast')
def convert(obj_var, native_var):
    obj_var = native_var
    native_var = obj_var
    return native_var

@autojit(locals=dict(obj=object_))
def convert_float(obj):
    var = float_(obj)
    return object_(var)

@autojit(locals=dict(obj=object_))
def convert_numeric(obj, dst_type):
    var = dst_type(obj)
    return object_(var)

@autojit
def convert_to_pointer(array):
    p = array.data
    return object_(p)

class TestConversion(unittest.TestCase):
    def test_conversion(self):
        assert convert(object(), 10.2) == 10.2
        assert convert(object(), 10) == 10
        assert convert(object(), "foo") == "foo"
        obj = object()
        assert convert(object(), obj) == obj
        assert convert(object(), 10.2 + 5j) == 10.2 + 5j

        assert convert_float(10.5) == 10.5

    def test_numeric_conversion(self):
        types = [
            char,
            uchar,
            short,
            ushort,
            int_,
            uint,
            long_,
            ulong,
            longlong,
            ulonglong,
            Py_ssize_t,
            size_t,
            float_,
            double,
#            longdouble,
            complex64,
            complex128,
        ]
        value = 2.5
        for dst_type in types:
            # print dst_type
            if dst_type.is_int:
                expected = 2
            else:
                expected = 2.5

            result = convert_numeric(value, dst_type)
            assert result == expected, (result, dst_type)

    def test_pointer_conversion(self):
        type = double.pointer()
        array = np.arange(10, dtype=np.double)
        # p = array.ctypes.data_as(type.to_ctypes())
        result = convert_to_pointer(array)
        assert ctypes.cast(result, ctypes.c_void_p).value == array.ctypes.data

if __name__ == "__main__":
#    TestConversion("test_pointer_conversion").test_pointer_conversion()
    from numba.testing import test_support
    test_support.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.nodes import *

basic_block_fields = ['cond_block', 'if_block', 'else_block', 'exit_block']

def delete_control_blocks(flow_node, flow):
    """
    Remove all control flow basic blocks from the CFG given a FlowNode
    and the CFG. Also removes Name references from cf_references.
    """
    parent = flow_node.cond_block.idom
    flow_node.exit_block.reparent(parent)
    flow.blocks.remove(flow_node.exit_block)
    flow_node.exit_block = None

    #flow_node.cond_block.delete(flow)
    #flow_node.if_block.delete(flow)

    #if flow_node.orelse:
    #    flow_node.else_block.delete(flow)

    from numba import control_flow
    control_flow.DeleteStatement(flow).visit(flow_node)

class FlowNode(Node):
    """
    Node that has control flow basic blocks.
    """

    cond_block = None
    if_block = None
    else_block = None
    exit_block = None

    def __init__(self, **kwargs):
        super(FlowNode, self).__init__(**kwargs)

        from numba import control_flow
        for field_name in basic_block_fields:
            if not getattr(self, field_name):
                block = control_flow.ControlBlock(-1, is_fabricated=True)
                setattr(self, field_name, block)


class If(ast.If, FlowNode):
    "An if statement node. Has the basic block attributes from FlowNode"

class While(ast.While, FlowNode):
    "A while loop node. Has the basic block attributes from FlowNode"

    # Place to jump to when we see a 'continue'. The default is
    # 'the condition block'. For 'for' loops we set this to
    # 'the counter increment block'
    continue_block = None

class For(ast.For, FlowNode):
    "A for loop node. Has the basic block attributes from FlowNode"

def merge_cfg_in_ast(basic_block_fields, bodies, node):
    """
    Merge CFG blocks into the AST. E.g.

        While(test=x, body=y)

    becomes

        While(test=ControlBlock(0, body=[x]), body=ControlBlock(1, body=[y]))
    """
    for bb_name, body_name in zip(basic_block_fields, bodies):
        body = getattr(node, body_name)
        bb = getattr(node, bb_name)

        if not body:
            continue

        # Merge AST child in body list of CFG block
        if isinstance(body, list):
            bb.body = body
            bb = [bb]
        else:
            bb.body = [body]

        # Set basic block as an AST child of the node
        setattr(node, body_name, bb)

def merge_cfg_in_while(node):
    bodies = ['test', 'body', 'orelse']
    merge_cfg_in_ast(basic_block_fields, bodies, node)

def build_if(cls=If, **kwargs):
    node = cls(**kwargs)
    merge_cfg_in_while(node)
    return node

def build_while(**kwargs):
    return build_if(cls=While, **kwargs)

def build_for(**kwargs):
    result = For(**kwargs)
    merge_cfg_in_ast(basic_block_fields, ['iter', 'body', 'orelse'], result)
    merge_cfg_in_ast(['target_block'], ['target'], result)
    return result

def if_else(op, cond_left, cond_right, lhs, rhs):
    "Implements 'lhs if cond_left <op> cond_right else rhs'"
    test = ast.Compare(left=cond_left, ops=[op],
                       comparators=[cond_right])
    test.right = cond_right
    test = typednode(test, bool_)

    return build_if(test=test, body=[lhs], orelse=[rhs] if rhs else [])

class LowLevelBasicBlockNode(Node):
    """
    Evaluate a statement or expression in a new LLVM basic block.
    """

    _fields = ['body']

    def __init__(self, body, label='unnamed', **kwargs):
        super(LowLevelBasicBlockNode, self).__init__(**kwargs)
        self.body = body
        self.label = label
        self.entry_block = None

    def create_block(self, translator, label=None):
        if self.entry_block is None:
            self.entry_block = translator.append_basic_block(label or self.label)
        return self.entry_block

class MaybeUnusedNode(Node):
    """
    Wraps an ast.Name() to indicate that the result may be unused.
    """

    _fields = ["name_node"]

    def __init__(self, name_node):
        self.name_node = name_node



# -*- coding: utf-8 -*-

"""
Doctest support exposed to numba users.
"""

from __future__ import print_function, division, absolute_import

import sys
import doctest

from numba.testing import doctest_support

doctest_options = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE

def testmod(module=None, run=True, optionflags=0, verbosity=2):
    """
    Tests a doctest modules with numba functions. When run in nosetests, only
    populates module.__test__, when run as main, runs the doctests.

    module: the module to run the doctests in
    run: whether to run the doctests or just build a __test__ dict
    verbosity: verbosity level passed to unittest.TextTestRunner

        The defualt is 2

    optionflags: doctest options (e.g. doctest.ELLIPSIS)
    """
    if module is None:
        mod_globals = sys._getframe(1).f_globals
        modname = mod_globals['__name__']
        module = __import__(modname)

    doctest_support.testmod(
        module,
        run_doctests=run,
        optionflags=optionflags,
        verbosity=verbosity,
    )



import sys
import ast
import pprint

from numba import error

def iter_fields(node):
    """
    Yield a tuple of ``(fieldname, value)`` for each field in ``node._fields``
    that is present on *node*.
    """
    for field in node._fields:
        try:
            yield field, getattr(node, field)
        except AttributeError:
            pass



class With(ast.stmt):
    """
    Node for AST compatibility with python 3.3.
    """

    _fields = ['context_expr', 'optional_vars', 'body']


class AST3to2(ast.NodeTransformer):

    def _visit_list(self, alist):
        new_values = []
        for value in alist:
            if isinstance(value, ast.AST):
                value = self.visit(value)
                if value is None:
                    continue
                elif not isinstance(value, ast.AST):
                    new_values.extend(value)
                    continue
            new_values.append(value)
        return new_values

    def generic_visit(self, node):
        for field, old_value in iter_fields(node):
            old_value = getattr(node, field, None)
            if isinstance(old_value, list):
                old_value[:] = self._visit_list(old_value)
            elif isinstance(old_value, ast.AST):
                new_node = self.visit(old_value)
                if new_node is None:
                    delattr(node, field)
                else:
                    setattr(node, field, new_node)
        return node

    def __visit_FunctionDef(self, node):
        new_node = ast.FunctionDef(args=self.visit_arguments(node.args),
                                   body=self._visit_list(node.body),
                                   decorator_list=self._visit_list(node.decorator_list),
                                   name=node.name)
        ast.copy_location(new_node, node)
        return new_node

    def visit_Index(self, node):
        if isinstance(node.value, ast.Ellipsis):
            return node.value
        return node

    def visit_arguments(self, node):
        ret = []
        for arg_node in node.args:
            if isinstance(arg_node, ast.arg):
                new_node = ast.Name(ctx=ast.Param(), id=arg_node.arg)
                ret.append(new_node)
            elif isinstance(arg_node, ast.Name):
                ret.append(arg_node)
            else:
                raise TypeError('Cannot transform node %r' % arg_node)
        return ast.arguments(args=ret, defaults=node.defaults,
                             kwarg=node.kwarg, vararg=node.vararg)

    def visit_With(self, node):
        """
        Rewrite the With statement.

        Python < 3.3:

             With(expr context_expr, expr? optional_vars, stmt* body)

        Python 3.3:

            With(withitem* items, stmt* body)
            withitem = (expr context_expr, expr? optional_vars)
        """
        if sys.version_info[:2] >= (3, 3):
            if len(node.items) > 1:
                raise error.NumbaError(node,
                                       "Only one 'with' context is support")

            withitem = node.items[0]

            new_node = With()
            new_node.context_expr = withitem.context_expr
            new_node.optional_vars = withitem.optional_vars
            new_node.body = node.body

            node = ast.copy_location(new_node, node)

        self.generic_visit(node)
        return node


# -*- coding: utf-8 -*-
"""
Resolve calls to math functions.

During type inference this produces MathNode nodes, and during
final specialization it produces LLVMIntrinsicNode and MathCallNode
nodes.
"""
from __future__ import print_function, division, absolute_import

import math
import cmath
try:
    import __builtin__ as builtins
except ImportError:
    import builtins

import numpy as np

from numba import *
from numba import nodes
from numba.symtab import Variable
from numba.typesystem import get_type
from numba.type_inference.modules import utils


#----------------------------------------------------------------------------
# Utilities
#----------------------------------------------------------------------------

register_math_typefunc = utils.register_with_argchecking

def binop_type(context, x, y):
    "Binary result type for math operations"
    x_type = get_type(x)
    y_type = get_type(y)
    return context.promote_types(x_type, y_type)

#----------------------------------------------------------------------------
# Determine math functions
#----------------------------------------------------------------------------

# sin(double), sinf(float), sinl(long double)
unary_libc_math_funcs = [
    'sin',
    'cos',
    'tan',
    'sqrt',
    'acos',
    'asin',
    'atan',
    'atan2',
    'sinh',
    'cosh',
    'tanh',
    'asinh',
    'acosh',
    'atanh',
    'log',
    'log2',
    'log10',
    'fabs',
    'erfc',
    'ceil',
    'exp',
    'exp2',
    'expm1',
    'rint',
    'log1p',
]

n_ary_libc_math_funcs = [
    'pow',
    'round',
]

all_libc_math_funcs = unary_libc_math_funcs + n_ary_libc_math_funcs

#----------------------------------------------------------------------------
# Math Type Inferers
#----------------------------------------------------------------------------

# TODO: Move any rewriting parts to lowering phases

def infer_unary_math_call(context, call_node, arg, default_result_type=double):
    "Resolve calls to math functions to llvm.log.f32() etc"
    # signature is a generic signature, build a correct one
    type = get_type(call_node.args[0])

    if type.is_numeric and type.kind < default_result_type.kind:
        type = default_result_type
    elif type.is_array and type.dtype.is_int:
        type = type.copy(dtype=double)

    # signature = minitypes.FunctionType(return_type=type, args=[type])
    # result = nodes.MathNode(py_func, signature, call_node.args[0])
    nodes.annotate(context.env, call_node, is_math=True)
    call_node.variable = Variable(type)
    return call_node

def infer_unary_cmath_call(context, call_node, arg):
    result = infer_unary_math_call(context, call_node, arg,
                                   default_result_type=complex128)
    nodes.annotate(context.env, call_node, is_cmath=True)
    return result

# ______________________________________________________________________
# pow()

def pow_(context, call_node, node, power, mod=None):
    dst_type = binop_type(context, node, power)
    call_node.variable = Variable(dst_type)
    return call_node

register_math_typefunc((2, 3), math.pow)
register_math_typefunc(2, np.power)

# ______________________________________________________________________
# abs()

def abs_(context, node, x):
    import builtinmodule

    argtype = get_type(x)

    if argtype.is_array and argtype.is_numeric:
        # Handle np.abs() on arrays
        dtype = builtinmodule.abstype(argtype.dtype)
        result_type = argtype.copy(dtype=dtype)
        node.variable = Variable(result_type)
        return node

    return builtinmodule.abs_(context, node, x)

register_math_typefunc(1, np.abs)

#----------------------------------------------------------------------------
# Register Type Functions
#----------------------------------------------------------------------------

def register_math(nargs, value):
    register = register_math_typefunc(nargs)
    register(infer_unary_math_call, value)

def register_cmath(nargs, value):
    register = register_math_typefunc(nargs)
    register(infer_unary_cmath_call, value)

def register_typefuncs():
    modules = [builtins, math, cmath, np]
    # print all_libc_math_funcs
    for libc_math_func in unary_libc_math_funcs:
        for module in modules:
            if hasattr(module, libc_math_func):
                if module is cmath:
                    register = register_cmath
                else:
                    register = register_math

                register(1, getattr(module, libc_math_func))

register_typefuncs()



"""
>>> compile_class(False).__name__
'Base'
>>> compile_class(True).__name__
Warning ...: Unused argument '...'
Warning ...: Unused argument '...'
'Base'
"""

from numba import *

def compile_class(warn):
    @jit(warn=warn, warnstyle='simple') # TODO: only issue error once !
    class Base(object):

        @void(int_)
        def method(self, argument):
            pass

    return Base

if __name__ == '__main__':
    # compile_class(True)
    import numba
    numba.testing.testmod()



from itertools import cycle

from numba import *
from numba.testing.test_support import parametrize, main

import numpy as np

# NOTE: We make these two separate classes because we don't want a set of
# NOTE: precompiled methods to affect the tests

del python, nopython # Make sure we run in *compiled* mode

def _make_list_func(self, A):
    L = []

    with nopython:
        for i in range(A.shape[0]):
            item = A[i]

            with python:
                L.append(item)

    return L

def make_list_func(self, A):
    return self._make_list(A)

@autojit
class Base1(object):
    """
    Test numba calling autojit methods
    """
    def __init__(self, myfloat):
        self.value = myfloat

    def getvalue(self):
        return self.value

    _make_list = _make_list_func
    make_list = make_list_func

@autojit
class Base2(object):
    """
    Test numba calling autojit methods
    """
    def __init__(self, myfloat):
        self.value = myfloat

    def getvalue(self):
        return self.value

    _make_list = _make_list_func
    make_list = make_list_func

@autojit
class Base3(object):
    """
    Test Python calling autojit methods.
    """

    def __init__(self, myfloat):
        self.value = myfloat

    def getvalue(self):
        return self.value

    make_list = _make_list_func

@autojit
def run(obj, array):
    list = obj.make_list(array)
    return list

#------------------------------------------------------------------------
# Tests
#------------------------------------------------------------------------

obj1 = Base1(10.0)
obj2 = Base2(10.0)
obj3 = Base3(10.0)

assert obj1.value == obj2.value == obj3.value == 10.0

dtypes = (
    np.float32, np.int32,
    np.double, np.int64,
    np.complex64, np.complex128,
)

params = (list(zip(cycle([obj1]), dtypes)) +
          list(zip(cycle([obj3]), dtypes)))

# ______________________________________________________________________
# Parameterized tests

@parametrize(*params)
def test_python_specialize_method(param):
    obj, dtype = param

    A = np.arange(10, dtype=dtype)
    L = obj.make_list(A)
    assert np.all(A == L)

@parametrize(*zip(cycle([obj2]), dtypes))
def test_numba_func_use_method(param):
    obj, dtype = param

    A = np.arange(10, dtype=dtype)
    L = run(obj, A)
    assert np.all(A == L)


if __name__ == '__main__':
    # obj, dtype = obj2, np.double
    #
    # A = np.arange(10, dtype=dtype)
    # L = run(obj, A)
    # assert np.all(A == L)

    main()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import copy
import types
import ctypes
from functools import reduce

import llvm.core
import numpy as np

import numba
from numba.support.ctypes_support import is_ctypes, from_ctypes_value
from numba.support import cffi_support
from numba.minivect.minitypes import map_dtype, object_
import numba.minivect.minitypes

from numba import numbawrapper
from numba.typesystem import *
import numba.typesystem

def is_dtype_constructor(value):
    return isinstance(value, type) and issubclass(value, np.generic)

def infer_container_type(typemapper, value):
    assert isinstance(value, (tuple, list, dict))

    if isinstance(value, tuple):
        container_type = TupleType
    elif isinstance(value, list):
        container_type = ListType
    else:
        key_type = infer_container_type(typemapper, value.keys())
        value_type = infer_container_type(typemapper, value.values())
        return DictType(key_type, value_type, size=len(value))

    if len(value) < 30:
        # Figure out base type if the container is not too large
        unify = typemapper.promote_types
        base_type = reduce(unify, (typemapper.from_python(child)
                                   for child in value))
    else:
        base_type = object_

    return container_type(base_type, size=len(value))


class NumbaTypeMapper(minitypes.TypeMapper):
    """
    Map types from Python values, handle type promotions, etc
    """

    def __init__(self, context):
        super(NumbaTypeMapper, self).__init__(context)
        # self.ctypes_func_type = type(ctypes.CFUNCTYPE(ctypes.c_int))
        # libc = ctypes.CDLL(ctypes.util.find_library('c'))
        # self.ctypes_func_type2 = type(libc.printf)

    def to_llvm(self, type):
        from numba import llvm_types
        if type.is_array:
            return llvm_types._numpy_array
        elif type.is_complex:
            lbase_type = type.base_type.to_llvm(self.context)
            return llvm.core.Type.struct([lbase_type, lbase_type])
        elif type.is_py_ssize_t:
            return llvm_types._llvm_py_ssize_t
        elif type.is_object:
            return llvm_types._pyobject_head_struct_p

        return super(NumbaTypeMapper, self).to_llvm(type)

    def from_python(self, value):
        from numba.type_inference import module_type_inference

        if isinstance(value, np.ndarray):
            dtype = map_dtype(value.dtype)
            return minitypes.ArrayType(dtype, value.ndim) #,
                                       #is_c_contig=value.flags['C_CONTIGUOUS'],
                                       #is_f_contig=value.flags['F_CONTIGUOUS'])
        elif isinstance(value, np.dtype):
            return numba.typesystem.from_numpy_dtype(value)
        elif is_dtype_constructor(value):
            return numba.typesystem.from_numpy_dtype(np.dtype(value))
        elif isinstance(value, (tuple, list, dict)):
            return infer_container_type(self, value)
        elif isinstance(value, types.ModuleType):
            return ModuleType(value)
        # elif isinstance(value, (self.ctypes_func_type, self.ctypes_func_type2)):
        elif is_ctypes(value):
            result = from_ctypes_value(value)
            if result.is_function:
                pointer = ctypes.cast(value, ctypes.c_void_p).value
                return PointerFunctionType(value, pointer, result)
            else:
                return result
        elif cffi_support.is_cffi_func(value):
            signature = cffi_support.get_signature(value)
            pointer = cffi_support.get_pointer(value)
            return PointerFunctionType(value, pointer, signature)
        elif isinstance(value, minitypes.Type):
            return CastType(dst_type=value)
        elif hasattr(type(value), '__numba_ext_type'):
            return getattr(type(value), '__numba_ext_type')
        elif value is numba.NULL:
            return null_type
        elif numbawrapper.is_numba_wrapper(value):
            return JitType(value)
        elif isinstance(value, numbawrapper.NumbaSpecializingWrapper):
            return AutojitType(value)
        elif hasattr(value, 'from_address') and hasattr(value, 'in_dll'):
            # Try to detect ctypes pointers, or default to minivect
            try:
                ctypes.cast(value, ctypes.c_void_p)
            except ctypes.ArgumentError:
                pass
            else:
                pass
                #type = convert_from_ctypes(value)
                #value = ctypes.cast(value, ctypes.c_void_p).value
                #return CTypesPointerType(type, valuee

        result_type = super(NumbaTypeMapper, self).from_python(value)

        if result_type == object_ and module_type_inference.is_registered(value):
            result = module_type_inference.module_attribute_type(value)
            if result is not None:
                result_type = result
            else:
                result_type = KnownValueType(value)

        return result_type

    def promote_types(self, type1, type2, assignment=False):
        have = lambda p1, p2: have_properties(type1, type2, p1, p2)

        if (type1.is_array or type2.is_array) and not \
            (type1.is_array and type2.is_array):
            if type1.is_array:
                array_type = type1
                other_type = type2
            else:
                array_type = type2
                other_type = type1

            type = copy.copy(array_type)
            type.dtype = self.promote_types(array_type.dtype, other_type)

            if type.dtype.is_object and not array_type.dtype.is_object:
                # Make sure that (double[:], object_) -> object_
                type = object_
            return type
        elif type1.is_unresolved or type2.is_unresolved:
            if type1.is_unresolved:
                type1 = type1.resolve()
            if type2.is_unresolved:
                type2 = type2.resolve()

            if type1.is_unresolved or type2.is_unresolved:
                # The Variable is really only important for ast.Name, fabricate
                # one
                from numba import symtab
                var = symtab.Variable(None)
                return PromotionType(var, self.context, [type1, type2])
            else:
                return self.promote_types(type1, type2)
        elif have("is_pointer", "is_null"):
            return [type1, type2][type1.is_null] # return the pointer type
        elif have("is_pointer", "is_int"):
            return [type1, type2][type1.is_int] # return the pointer type

        return super(NumbaTypeMapper, self).promote_types(type1, type2)


def have_properties(type1, type2, property1, property2):
    """
    Return whether the two types satisfy the two properties:

    >>> have_properties(int32, int32.pointer(), "is_pointer", "is_int")
    True
    """
    type1_p1 = getattr(type1, property1)
    type1_p2 = getattr(type1, property2)
    type2_p1 = getattr(type2, property1)
    type2_p2 = getattr(type2, property2)

    if (type1_p1 and type2_p2) or (type1_p2 and type2_p1):
        if type1_p1:
            return type1
        else:
            return type2
    else:
        return None



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba import *
from numba import llvm_types
from numba import typedefs
from numba.utility.cbuilder.library import register
from numba.utility.cbuilder.numbacdef import NumbaCDefinition, from_numba

from llvm_cbuilder import shortnames

#------------------------------------------------------------------------
# Utilities
#------------------------------------------------------------------------

p_py_ssize_t = shortnames.pointer(shortnames.py_ssize_t)

def ob_refcnt(obj_p):
    return deref(p_refcnt(obj_p))

def p_refcnt(obj_p):
    return obj_p.cast(p_py_ssize_t)

def deref(obj_p):
    return obj_p[0]

def const(ctemp, val):
    return ctemp.parent.constant(shortnames.py_ssize_t, val)

def add_refcnt(obj_p, refcnt):
    refcnt = const(obj_p, refcnt)
    refs = ob_refcnt(obj_p)
    refs += refcnt

def not_null(ptr):
    return ptr.cast(shortnames.py_ssize_t) != const(ptr, 0)

#------------------------------------------------------------------------
# Base Refcount Class
#------------------------------------------------------------------------

# TODO: Support tracing refcount operations (debug mode)
# TODO: Support refcount error checking (testing/debug mode)

class Refcounter(NumbaCDefinition):

    def set_signature(self, env, context):
        PyObject = typedefs.PyObject_HEAD

        self._argtys_ = [
            ('obj', PyObject.pointer().to_llvm(context)),
        ]
        self._retty_ = shortnames.void

#------------------------------------------------------------------------
# Refcount Implementations
#------------------------------------------------------------------------

@register
class Py_INCREF(Refcounter):
    "LLVM inline version of Py_INCREF"

    def body(self, obj):
        add_refcnt(obj, 1)
        self.ret()

@register
class Py_DECREF(Refcounter):
    "LLVM inline version of Py_DECREF"

    def body(self, obj):
        refcnt = ob_refcnt(obj)
        one = self.constant(refcnt.type, 1)
        Py_DecRef = self.external_cfunc('Py_DecRef')

        with self.ifelse(refcnt > one) as ifelse:

            with ifelse.then():
                # ob_refcnt > 1, just decrement
                add_refcnt(obj, -1)

            with ifelse.otherwise():
                # ob_refcnt == 1, dealloc
                Py_DecRef(obj)

        self.ret()

@register
class Py_XINCREF(Refcounter):
    "LLVM inline version of Py_XINCREF"

    def body(self, obj):
        with self.ifelse(not_null(obj)) as ifelse:
            with ifelse.then():
                add_refcnt(obj, 1)

        self.ret()

@register
class Py_XDECREF(Refcounter):
    "LLVM inline version of Py_XDECREF"

    def body(self, obj):
        py_decref = self.cbuilder_cfunc(Py_DECREF)

        with self.ifelse(not_null(obj)) as ifelse:
            with ifelse.then():
                py_decref(obj)

        self.ret()



# -*- coding: utf-8 -*-

"""
>>> tuple_unpacking_error(2)
Traceback (most recent call last):
    ...
NumbaError: ...: Only NumPy attributes and list or tuple literals can currently be unpacked
"""

from __future__ import print_function, division, absolute_import

import numba
from numba import *
import numpy as np

@autojit
def tuple_unpacking_error(obj):
    a, b = obj

if __name__ == "__main__":
    numba.testing.testmod()



"""
Test support for ctypes. See also numba.tests.foreign_call.test_ctypes_call.
"""

import ctypes

from numba import *
from numba import environment
from numba.support import ctypes_support

try:
    from numba.tests.support import ctypes_values
except ImportError:
    ctypes_values = None

#-------------------------------------------------------------------
# Utilities
#-------------------------------------------------------------------

env = environment.NumbaEnvironment.get_environment()
context = env.context
from_python = context.typemapper.from_python

def get_cast_type(type):
    assert type.is_cast
    return type.dst_type

def assert_signature(ctypes_func, expected=None):
    sig = from_python(ctypes_func)
    assert sig.is_pointer_to_function
    if expected:
        assert sig.signature == expected, (sig.signature, expected)

#-------------------------------------------------------------------
# Tests
#-------------------------------------------------------------------

if ctypes_values:
    rk_state_t = get_cast_type(from_python(ctypes_values.rk_state))

def ctypes_func_values():
    int_or_long = long_ if ctypes.c_int == ctypes.c_long else int_
    long_or_longlong = (longlong if ctypes.c_long == ctypes.c_longlong
                        else long_)

    signature = int_or_long(rk_state_t.pointer())
    assert_signature(ctypes_values.rk_randomseed, signature)

    signature = void(long_or_longlong, rk_state_t.pointer())
    assert_signature(ctypes_values.rk_seed, signature)

    signature = double(rk_state_t.pointer(), double, double)
    assert_signature(ctypes_values.rk_gamma, signature)

def ctypes_data_values():
    assert from_python(ctypes_values.state) == rk_state_t
    assert from_python(ctypes_values.state_p) == rk_state_t.pointer()
    assert from_python(ctypes_values.state_vp) == void.pointer()
    assert from_python(ctypes.c_void_p(10)) == void.pointer()

    ctypes_double_p = ctypes.POINTER(ctypes.c_double)(ctypes.c_double(10))
    assert from_python(ctypes_double_p) == double.pointer()

def test():
    if ctypes_values is not None:
        ctypes_func_values()
        ctypes_data_values()


if __name__ == '__main__':
    test()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

import numpy as np
from numba import autojit, jit, double, void, uint32, npy_intp, typeof

def uint_int_div_ary(elts, normdist, seed):
    for i in xrange(elts.shape[0]):
        # Problem with using sext instead of zext for uint32
        elt = (seed[i] // uint32(normdist.shape[0]))
        elts[i] = elt

def test_uint_int_div_ary():
    NPATHS = 10
    normdist = np.empty(1000) #np.random.normal(0., 1., 1000)
    seed = np.arange(0x80000000, 0x80000000 + NPATHS, dtype=np.uint32)

    gold = np.empty(NPATHS, dtype=np.int32)
    got = gold.copy()
    uint_int_div_ary(gold, normdist, seed)

    print('expect %s' % gold)
    sig = void(uint32[:], double[:], uint32[:])
    numba_func = jit(sig)(uint_int_div_ary)
    numba_func(got, normdist, seed)
    print('got %s' % got)

    assert all(gold == got)

if __name__ == '__main__':
    test_uint_int_div_ary()



# -*- coding: utf-8 -*-
"""
Code writers and formatters. Subclass CodeWriter to suit the needs of
a certain code generator backend.
"""
from __future__ import print_function, division, absolute_import

try:
    from Cython.Compiler import Tempita as tempita
except ImportError:
    try:
        import tempita
    except ImportError:
        tempita = None

class CodeWriter(object):
    """
    Write code as objects for later assembly.

    .. attribute:: loop_levels

        CodeWriter objects just before the start of each loop

    .. attribute:: tiled_loop_levels

        same as loop_levels, but takes into account tiled loop patterns

    .. attribute:: cleanup_levels

        CodeWriter objects just after the end of each loop

    .. attribute:: declaration_levels

        same as loop_levels, but a valid insertion point for C89 declarations
    """

    error_handler = None

    def __init__(self, context, buffer=None):
        self.buffer = buffer or _CodeTree()
        self.context = context

        self.loop_levels = []
        self.tiled_loop_levels = []
        self.declaration_levels = []

    @classmethod
    def clone(cls, other, context, buffer):
        return cls(context, buffer)

    def insertion_point(self):
        """
        Create an insertion point for the code writer. Any code written
        to this insertion point (later on) is inserted in the output code at
        the point where this method was called.
        """
        result = self.clone(self, self.context, self.buffer.insertion_point())
        result.loop_levels = list(self.loop_levels)
        result.tiled_loop_levels = list(self.tiled_loop_levels)
        result.declaration_levels = list(self.declaration_levels)
        return result

    def write(self, value):
        self.buffer.output.append(value)

    def put_label(self, label):
        "Insert a label in the code"
        self.write(label)

    def put_goto(self, label):
        "Jump to a label. Implement in subclasses"

class CCodeWriter(CodeWriter):
    """
    Code writer to write C code. Has both a prototype buffer and an
    implementation buffer. The prototype buffer will contain the C
    prototypes, and the implementation buffer the actual function
    code.
    """

    def __init__(self, context, buffer=None, proto_code=None):
        super(CCodeWriter, self).__init__(context, buffer)
        if proto_code is None:
            self.proto_code = type(self)(context, proto_code=False)
        self.indent = 0

    def put_label(self, label):
        "Insert a C label"
        self.putln('%s:' % self.mangle(label.name))

    def put_goto(self, label):
        "Jump to (goto) a label"
        self.putln("goto %s;" % self.mangle(label.name))

    def putln(self, s):
        "Write a code string as a line. Also performs indentation"
        self.indent -= s.count('}')
        self.write("%s%s\n" % (self.indent * '    ', s))
        self.indent += s.count('{')

    def mangle(self, s):
        "Mangle symbol names"
        return "__mini_mangle_%s" % s

    @classmethod
    def clone(cls, other, context, buffer):
        result = super(CCodeWriter, cls).clone(other, context, buffer)
        result.indent = other.indent
        return result

def sub_tempita(s, context, file=None, name=None):
    "Run the tempita template engine on string the given string."
    if not s:
        return None

    if file:
        context['__name'] = "%s:%s" % (file, name)
    elif name:
        context['__name'] = name

    if tempita is None:
        raise RuntimeError("Tempita was not installed")

    return tempita.sub(s, **context)

class TempitaCodeWriter(CodeWriter):
    """
    Code writer which supports writing Tempita strings. See
    http://pythonpaste.org/tempita/ for documentation on Tempita.
    """

    def putln(self, string, context_dict):
        self.write(sub_tempita(string) + '\n')

class CodeFormatter(object):
    """
    Default code formatting, which returns the formatted code as a list
    of objects (the ones written to the :py:class:`minivect.codegen.CodeWriter`)
    """
    def format(self, codewriter):
        return codewriter.buffer.getvalue()

class CodeStringFormatter(CodeFormatter):
    "Format code as strings"
    def format(self, codewriter):
        return "".join(codewriter.buffer.getvalue())

class CCodeStringFormatter(CodeStringFormatter):
    "Format the prototype and code implementation"

    def format(self, codewriter):
        return ("".join(codewriter.proto_code.buffer.getvalue()),
                "".join(codewriter.buffer.getvalue()))

class _CodeTree(object):
    """
    See Cython/StringIOTree
    """

    def __init__(self, output=None, condition=None):
        self.prepended_children = []
        self.output = output or []

    def _getvalue(self, result):
        for child in self.prepended_children:
            child._getvalue(result)
        result.extend(self.output)

    def getvalue(self):
        result = []
        self._getvalue(result)
        return result

    def clone(self, output=None):
        return type(self)(output)

    def commit(self):
        if self.output:
            self.prepended_children.append(self.clone(self.output))
            self.output = []

    def insertion_point(self):
        self.commit()
        ip = self.clone()
        self.prepended_children.append(ip)
        return ip



cdef extern from "pstdint.h":
    ctypedef unsigned int uint32_t
    ctypedef unsigned long long uint64_t
    ctypedef unsigned short uint16_t
    ctypedef unsigned char uint8_t
    ctypedef uint64_t uintptr_t

cdef extern from "perfecthash.h":
    ctypedef struct PyCustomSlots_Entry:
        uint64_t id
        void *ptr

    ctypedef struct PyCustomSlots_Table:
        uint64_t flags
        uint64_t m_f, m_g
        PyCustomSlots_Entry *entries
        uint16_t n, b
        uint8_t r

    ctypedef struct PyCustomSlots_Table_64_64:
        PyCustomSlots_Table base
        uint16_t d[64]
        PyCustomSlots_Entry entries_mem[64]


    int PyCustomSlots_PerfectHash(PyCustomSlots_Table *table, uint64_t *hashes)




# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from .miniutils import ctypes
from .minitypes import *

### Taken from Numba ###

# NOTE: The following ctypes structures were inspired by Joseph
# Heller's response to python-list question about ctypes complex
# support.  In that response, he said these were only suitable for
# Linux.  Might our milage vary?

class ComplexMixin (object):
    def _get(self):
        # FIXME: Ensure there will not be a loss of precision here!
        return self._numpy_ty_(self.real + (self.imag * 1j))

    def _set(self, value):
        self.real = value.real
        self.imag = value.imag

    value = property(_get, _set)

    @classmethod
    def from_param(cls, param):
        ret_val = cls()
        ret_val.value = param
        return ret_val

    @classmethod
    def make_ctypes_prototype_wrapper(cls, ctypes_prototype):
        '''This is a hack so that functions that return a complex type
        will construct a new Python value from the result, making the
        Numba compiled function a drop-in replacement for a Python
        function.'''
        # FIXME: See if there is some way of avoiding this additional
        # wrapper layer.
        def _make_complex_result_wrapper(in_func):
            ctypes_function = ctypes_prototype(in_func)
            def _complex_result_wrapper(*args, **kws):
                # Return the value property, not the ComplexMixin
                # instance built by ctypes.
                result = ctypes_function(*args, **kws)
                return result.value
            return _complex_result_wrapper
        return _make_complex_result_wrapper

try:
    import numpy as np
except ImportError:
    pass
else:
    class Complex64(ctypes.Structure, ComplexMixin):
        _fields_ = [('real', ctypes.c_float), ('imag', ctypes.c_float)]
        _numpy_ty_ = np.complex64

    class Complex128(ctypes.Structure, ComplexMixin):
        _fields_ = [('real', ctypes.c_double), ('imag', ctypes.c_double)]
        _numpy_ty_ = np.complex128

    if hasattr(np, 'complex256'):
        class Complex256(ctypes.Structure, ComplexMixin):
            _fields_ = [('real', ctypes.c_longdouble), ('imag', ctypes.c_longdouble)]
            _numpy_ty_ = np.complex256
    else:
        Complex256 = None

### End Taken from Numba ###



#! /usr/bin/env python
# ______________________________________________________________________
'''test_if

Test phi node (or similar) generation for CFG joins beyond
if-then-else statements.
'''
# ______________________________________________________________________
from __future__ import print_function
import sys
from numba import *

import unittest

# ______________________________________________________________________

def if_fn_1(arg):
    if arg > 0.:
        result = 22.
    else:
        result = 42.
    return result


def if_fn_2(i, j):
    n = 5
    m = 5
    if j >= 1 and j < n - 1 and i >= 1 and i < m - 1:
        return i + j
    return 0xcafe

def if_fn_3(i, j):
    n = 5
    m = 5
    if j >= 1:
        if j < n - 1:
            if i >= 1:
                if i < m - 1:
                    return i + j
    return 0xbeef

def if_fn_4(i, j):
    if i < 0 or j < 0:
        return i + j
    return 0xdead

def if_fn_5(i, j):
    if i < 0:
        return i + j
    if j < 0:
        return i + j
    return 0xdefaced

def if_fn_6(i, j):
    if i < j:
        return  #
    i += j

def if_fn_7(i):
    if i:
        return i + 1
    return i

def if_fn_8(i, j):
    if i > j:
        return 1
    return 0

def if_fn_9(i, j, k):
    if i or (j and k):
        return 1
    return 0

@autojit
def if_bool(b):
    if b:
        return 1
    else:
        return 2

@autojit
def if_bool_constant_true():
    if True:
        return 1
    else:
        return 2

@autojit
def if_bool_constant_false():
    if False:
        return 1
    else:
        return 2


# ______________________________________________________________________

class TestIf(unittest.TestCase):
    def test_if_fn_1(self):
        if_fn_1c = jit(restype=f4, argtypes=[f4], backend='ast')(if_fn_1)
        oracle = if_fn_1
        self.assertEqual(if_fn_1c(-1.), if_fn_1(-1.))
        self.assertEqual(if_fn_1c(1.),  if_fn_1(1.))

    def test_if_fn_2(self):
        if_fn_2c = jit(restype=i4, argtypes=[i4, i4], backend='ast')(if_fn_2)

        oracle = if_fn_2
        for i in range(6):
            for j in range(6):
                self.assertEqual(if_fn_2c(i, j), oracle(i, j))

    def test_if_fn_3(self):
        if_fn_3c = jit(restype=i4, argtypes=[i4, i4], backend='ast')(if_fn_3)

        oracle = if_fn_3
        for i in range(6):
            for j in range(6):
                self.assertEqual(if_fn_3c(i, j), oracle(i, j))

    def test_if_fn_4(self):
        try:
            if sys.version_info[:2] < (2, 7):
                raise ImportError
            from meta.decompiler import decompile_func
        except ImportError:
            print("Skipping if test, meta not available", file=sys.stderr)
        else:
            import ast, inspect
            if_fn_4c = jit(restype=i4, argtypes=[i4, i4], backend='ast')(if_fn_4)

            oracle = if_fn_4
            for i in range(-3, 3):
                for j in range(-3, 3):
                    self.assertEqual(if_fn_4c(i, j), oracle(i, j))

    def test_if_fn_5(self):
        if_fn_5c = jit(restype=i4, argtypes=[i4, i4], backend='ast')(if_fn_5)
        oracle = if_fn_5
        for i in range(-3, 3):
            for j in range(-3, 3):
                self.assertEqual(if_fn_5c(i, j), oracle(i, j))

    def test_if_fn_6(self):
        if_fn_6c = jit(restype=void, argtypes=[i4, i4], backend='ast')(if_fn_6)

    def test_if_fn_7(self):
        # if_fn_7c = jit(restype=i4, argtypes=[i4], backend='ast')(if_fn_7)
        if_fn_7c = autojit(if_fn_7)
        oracle = if_fn_7
        for i in range(-3, 3):
            self.assertEqual(if_fn_7c(i), oracle(i))
            self.assertEqual(if_fn_7c(float(i)), oracle(float(i)))
            # self.assertEqual(if_fn_7c(i+1j), oracle(i+1j))

    def test_if_fn_8(self):
        if_fn_5c = jit(restype=i4, argtypes=[i4, i4], backend='ast')(if_fn_8)
        oracle = if_fn_8
        for i in range(-3, 3):
            for j in range(-3, 3):
                self.assertEqual(if_fn_5c(i, j), oracle(i, j))

    def test_if_fn_9(self):
        if_fn_5c = jit(restype=i4, argtypes=[i4, i4, i4], backend='ast')(
            if_fn_9)
        oracle = if_fn_9
        for i in range(-2, 2):
            for j in range(-2, 2):
                for k in range(-2, 2):
                    self.assertEqual(if_fn_5c(i, j, k), oracle(i, j, k))

    def test_if_bool(self):
        self.assertEqual(if_bool(True), 1)
        self.assertEqual(if_bool(False), 2)
        self.assertEqual(if_bool_constant_true(), 1)
        self.assertEqual(if_bool_constant_false(), 2)

# ______________________________________________________________________

if __name__ == "__main__":
#    if_fn_1c = jit(restype=f4, argtypes=[f4], backend='ast')(if_fn_1)
#    if_fn_4c = jit(restype=i4, argtypes=[i4, i4], backend='ast')(if_fn_4)
#    if_fn_5c = jit(restype=i4, argtypes=[i4, i4], backend='ast')(if_fn_5)
#    if_fn_6c = jit(restype=void, argtypes=[i4, i4], backend='ast')(if_fn_6)
#    if_fn_7c = jit(restype=i4, argtypes=[i4], backend='ast')(if_fn_7)
#    print if_fn_7c(-2), if_fn_7(-2)
#    print if_bool_constant_true()
    unittest.main()

# ______________________________________________________________________
# End of test_if.py



# -*- coding: utf-8 -*-
'''llvm_types

Utility module containing common (to Numba) LLVM types.
'''
from __future__ import print_function, division, absolute_import
# ______________________________________________________________________

import ctypes
import struct as struct_
import llvm.core as lc

from numba import utils
from numba.typedefs import _trace_refs_, PyObject_HEAD

import logging

logger = logging.getLogger(__name__)

# ______________________________________________________________________

_plat_bits = struct_.calcsize('@P') * 8

# Assuming sizeof(c_size_t) == sizeof(c_ssize_t) == sizeof(Py_ssize_t)...
_sizeof_py_ssize_t = ctypes.sizeof(
    getattr(ctypes, 'c_ssize_t', getattr(ctypes, 'c_size_t')))

_int1 = lc.Type.int(1)
_int8 = lc.Type.int(8)
_int8_star = lc.Type.pointer(_int8)
_int32 = lc.Type.int(32)
_int64 = lc.Type.int(64)
_llvm_py_ssize_t = lc.Type.int(_sizeof_py_ssize_t * 8)
_llvm_size_t = _llvm_py_ssize_t
_intp = lc.Type.int(_plat_bits)
_intp_star = lc.Type.pointer(_intp)
_void_star = lc.Type.pointer(lc.Type.int(8))
_void_star_star = lc.Type.pointer(_void_star)
_float = lc.Type.float()
_double = lc.Type.double()
_complex64 = lc.Type.struct([_float, _float])
_complex128 = lc.Type.struct([_double, _double])


def to_llvm(type):
    return type.to_llvm(utils.context)

_pyobject_head = [to_llvm(ty) for name, ty in PyObject_HEAD.fields]
_pyobject_head_struct = to_llvm(PyObject_HEAD)
_pyobject_head_struct_p = lc.Type.pointer(_pyobject_head_struct)

_head_len = len(_pyobject_head)
_numpy_struct = lc.Type.struct(_pyobject_head+\
      [_void_star,          # data
       _int32,              # nd
       _intp_star,          # dimensions
       _intp_star,          # strides
       _void_star,          # base
       _void_star,          # descr
       _int32,              # flags
       _void_star,          # weakreflist
      ])
_numpy_array = lc.Type.pointer(_numpy_struct)

_BASE_ARRAY_FIELD_OFS = len(_pyobject_head)

_numpy_array_field_ofs = {
    'data' : _BASE_ARRAY_FIELD_OFS,
    'ndim' : _BASE_ARRAY_FIELD_OFS + 1,
    'shape' : _BASE_ARRAY_FIELD_OFS + 2,
    'strides' : _BASE_ARRAY_FIELD_OFS + 3,
    'base' : _BASE_ARRAY_FIELD_OFS + 4,
    'descr' : _BASE_ARRAY_FIELD_OFS + 5,
}

def constant_int(value, type=_int32):
    return lc.Constant.int(type, value)

# ______________________________________________________________________

class _LLVMCaster(object):
    def __init__(self, builder):
        self.builder = builder

    def cast(self, lvalue, dst_ltype, *args, **kws):
        src_ltype = lvalue.type
        if src_ltype == dst_ltype:
            return lvalue
        return self.build_cast(self.builder, lvalue, dst_ltype, *args, **kws)

    def build_pointer_cast(_, builder, lval1, lty2):
        return builder.bitcast(lval1, lty2)

    def build_int_cast(_, builder, lval1, lty2, unsigned = False):
        width1 = lval1.type.width
        width2 = lty2.width
        ret_val = lval1
        if width2 > width1:
            if unsigned:
                ret_val = builder.zext(lval1, lty2)
            else:
                ret_val = builder.sext(lval1, lty2)
        elif width2 < width1:
            # JDR: Compromise here on logging level...
            logger.info("Warning: Perfoming downcast.  May lose information.")
            ret_val = builder.trunc(lval1, lty2)
        return ret_val

    def build_float_ext(_, builder, lval1, lty2):
        return builder.fpext(lval1, lty2)

    def build_float_trunc(_, builder, lval1, lty2):
        logger.info("Warning: Perfoming downcast.  May lose information.")
        return builder.fptrunc(lval1, lty2)

    def build_int_to_float_cast(_, builder, lval1, lty2, unsigned = False):
        ret_val = None
        if unsigned:
            ret_val = builder.uitofp(lval1, lty2)
        else:
            ret_val = builder.sitofp(lval1, lty2)
        return ret_val

    def build_float_to_int_cast(_, builder, lval1, lty2, unsigned = False):
        ret_val = None
        if unsigned:
            ret_val = builder.fptoui(lval1, lty2)
        else:
            ret_val = builder.fptosi(lval1, lty2)
        return ret_val

    CAST_MAP = {
        lc.TYPE_POINTER : build_pointer_cast,
        lc.TYPE_INTEGER: build_int_cast,

        (lc.TYPE_FLOAT, lc.TYPE_DOUBLE) : build_float_ext,
        (lc.TYPE_DOUBLE, lc.TYPE_FP128) : build_float_ext,

        (lc.TYPE_DOUBLE, lc.TYPE_FLOAT) : build_float_trunc,
        (lc.TYPE_FP128, lc.TYPE_DOUBLE) : build_float_trunc,

        (lc.TYPE_INTEGER, lc.TYPE_FLOAT) : build_int_to_float_cast,
        (lc.TYPE_INTEGER, lc.TYPE_DOUBLE) : build_int_to_float_cast,

        (lc.TYPE_FLOAT, lc.TYPE_INTEGER) : build_float_to_int_cast,
        (lc.TYPE_DOUBLE, lc.TYPE_INTEGER) : build_float_to_int_cast,

    }

    @classmethod
    def build_cast(cls, builder, lval1, lty2, *args, **kws):
        ret_val = lval1
        lty1 = lval1.type
        lkind1 = lty1.kind
        lkind2 = lty2.kind

        # This looks like the wrong place to enforce this
        # TODO: We need to pass in the numba types instead
        # if lc.TYPE_INTEGER in (lkind1, lkind2) and 'unsigned' not in kws:
        #     # Be strict about having `unsigned` define when
        #     # we have integer types
        #     raise ValueError("Unknown signedness for integer type",
        #                      '%s -> %s' % (lty1, lty2), args, kws)

        if lkind1 == lkind2:

            if lkind1 in cls.CAST_MAP:
                ret_val = cls.CAST_MAP[lkind1](cls, builder, lval1, lty2,
                                               *args, **kws)
            else:
                raise NotImplementedError(lkind1)
        else:
            map_index = (lkind1, lkind2)
            if map_index in cls.CAST_MAP:
                ret_val = cls.CAST_MAP[map_index](cls, builder, lval1, lty2,
                                                  *args, **kws)
            else:
                raise NotImplementedError('Unable to cast from %s to %s.' %
                                          (str(lty1), str(lty2)))
        return ret_val

# ______________________________________________________________________
# End of llvm_types.py



# Issue: #144

# Thanks to Neal Becker

import numpy as np
from numba import *

f8_array_ty = f8[:]

@jit
class fir (object):
    @void(f8[:])
    def __init__ (self, coef):
        self.coef = coef
        self.inp = np.zeros_like (coef)

    @void(f8_array_ty)
    def shift (self, u):
        size = self.inp.size
        n = len (u)
        for i in range (size-1-n, -1, -1):
            self.inp[i+n] = self.inp[i]
        self.inp[:n] = u

    @f8 ()
    def compute1(self):
        s = 0
        size = self.coef.size
        for i in range (size):
            s += self.inp[i] * self.coef[i]
        return s

    @f8(f8_array_ty)
    def shift_compute1 (self, u):
        self.shift (u)
        return self.compute1()

    @f8_array_ty(f8_array_ty)
    def compute (self, u):
        out = np.empty_like (u)
        size = u.size
        for i in range (size):
            out[i] = self.shift_compute1 (u[i:i+1])
        return out

    @f8_array_ty(f8_array_ty)
    def __call__ (self, u):
        return self.compute (u)

if __name__ == '__main__':
    coef = np.random.rand (16)
    filt = fir (coef)
    inp = np.random.rand (1000)

    x = inp[:10].copy()
    out = filt.compute (inp)
    assert np.all(x == inp[:10])



import numba
from numba import *
from numba.typesystem import typeset
from numba.environment import NumbaEnvironment

def s(type):
    return type(type, type)

def test_typeset_matching():
    context = NumbaEnvironment.get_environment().context

    numeric = typeset.typeset([int_, longlong])
    n = numeric(numeric, numeric)
    f = numba.floating(numba.floating, numba.floating)

    signatures = [n, f, object_(object_, object_)]
    ts = typeset.typeset(signatures)

    assert ts.find_match(context, [float_, float_]) == s(float_)
    assert ts.find_match(context, [float_, double]) == s(double)
    assert ts.find_match(context, [longdouble, float_]) == s(longdouble)

    assert ts.find_match(context, [int_, int_]) == s(int_)
    assert ts.find_match(context, [int_, longlong]) == s(longlong)
    assert ts.find_match(context, [short, int_]) == s(int_)

    assert ts.find_match(context, [short, ulonglong]) is None

if __name__ == '__main__':
    test_typeset_matching()



"""
Minimal traits implementation:

    @traits
    class MyClass(object):

        attr = Instance(SomeClass)
        my_delegation = Delegate('attr')
"""

import inspect

# from numba.utils import TypedProperty

def traits(cls):
    "@traits class decorator"
    for name, py_func in vars(cls).items():
        if isinstance(py_func, TraitBase):
            py_func.set_attr_name(name)

    return cls

class TraitBase(object):
    "Base class for traits"

    def __init__(self, value, doc=None):
        self.value = value
        self.doc = doc

    def set_attr_name(self, name):
        self.attr_name = name

class Delegate(TraitBase):
    """
    Delegate to some other object.
    """

    def __init__(self, value, delegate_attr_name=None, doc=None):
        super(Delegate, self).__init__(value, doc=doc)
        self.delegate_attr_name = delegate_attr_name

    def __get__(self, instance, owner):
        obj = getattr(instance, self.value)
        return getattr(obj, self.delegate_attr_name or self.attr_name)

    def __set__(self, instance, value):
        obj = getattr(instance, self.value)
        return setattr(obj, self.delegate_attr_name or self.attr_name, value)



#! /usr/bin/env python
# ______________________________________________________________________

from numba import *
from numba import c_string_type as cstring, int_

from nose.tools import nottest
from numba.testing import test_support

# ______________________________________________________________________

def convert(input_str):
    return int(input_str[0:5])

# ______________________________________________________________________

def fast_convert(input_str):
    with nopython:
        return int(input_str[0:5])

# ______________________________________________________________________

class TestCString(test_support.ASTTestCase):
    def test_convert(self, **kws):
        jit_convert = self.jit(argtypes = (cstring,), restype = int_, **kws)(
            convert)
        for exp in xrange(10):
            test_str = str(10 ** exp)
            self.assertEqual(jit_convert(test_str), convert(test_str))

    def test_convert_nopython(self, **kws):
        jit_convert = self.jit(argtypes = (cstring,), restype = int_, **kws)(
            fast_convert)
        for exp in xrange(10):
            test_str = str(10 ** exp)
            self.assertEqual(jit_convert(test_str), convert(test_str))

# ______________________________________________________________________

if __name__ == "__main__":
#    TestCString('test_convert').debug()
    test_support.main()

# ______________________________________________________________________
# End of test_cstring.py



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.nodes import *
import numba.nodes

def is_expr(node):
    if not isinstance(node, Node):
        return True
    return isinstance(node, ExprNode)

class Node(ast.AST):
    """
    Superclass for Numba AST nodes
    """

    _fields = []
    _attributes = ('lineno', 'col_offset')

    def __init__(self, **kwargs):
        vars(self).update(kwargs)


class ExprNode(Node):
    """
    Node that is an expression.
    """

    def _variable_get(self):
        if not hasattr(self, '_variable'):
            self._variable = Variable(self.type)

        return self._variable

    def _variable_set(self, variable):
        self._variable = variable

    variable = property(_variable_get, _variable_set)

    def coerce(self, dst_type):
        return numba.nodes.CoercionNode(self, dst_type)

    @property
    def cloneable(self):
        if isinstance(self, (CloneNode, CloneableNode)):
            return self
        return CloneableNode(self)


class Name(ast.Name, ExprNode):

    cf_maybe_null = False
    raise_unbound_node = None

    _fields = ast.Name._fields + ('check_unbound',)

    def __init__(self, id, ctx, *args, **kwargs):
        super(Name, self).__init__(*args, **kwargs)
        self.id = self.name = id
        self.ctx = ctx

    def __repr__(self):
        type = getattr(self, 'type', "")
        if type:
            type = ', %s' % type
        name = self.name
        if hasattr(self, 'variable') and self.variable.renamed_name:
            name = self.variable.unmangled_name
        return "name(%s%s)" % (name, type)

    def __deepcopy__(self, memo):
        result = Name(self.id, self.ctx)
        result.cf_maybe_null = self.cf_maybe_null
        result.raise_unbound_node = self.raise_unbound_node
        return result

class WithPythonNode(Node):
    "with python: ..."

    _fields = ['body']

class WithNoPythonNode(WithPythonNode):
    "with nopython: ..."


class CloneableNode(ExprNode):
    """
    Create a node that can be cloned. This allows sub-expressions to be
    re-used without re-evaluating them.
    """

    _fields = ['node']

    def __init__(self, node, **kwargs):
        super(CloneableNode, self).__init__(**kwargs)
        self.node = node
        self.clone_nodes = []
        self.type = node.type

    @property
    def clone(self):
        return CloneNode(self)

    def __repr__(self):
        return "cloneable(%s)" % self.node

class CloneNode(ExprNode):
    """
    Clone a CloneableNode. This allows the node's sub-expressions to be
    re-used without re-evaluating them.

    The CloneableNode must be evaluated before the CloneNode is evaluated!
    """

    _fields = ['node']

    def __init__(self, node, **kwargs):
        super(CloneNode, self).__init__(**kwargs)

        assert isinstance(node, CloneableNode)
        self.node = node
        self.type = node.type
        node.clone_nodes.append(self)

        self.llvm_value = None

    @property
    def clone(self):
        self

    def __repr__(self):
        return "clone(%s)" % self.node

class ExpressionNode(ExprNode):
    """
    Node that allows an expression to execute a bunch of statements first.
    """

    _fields = ['stmts', 'expr']

    def __init__(self, stmts, expr, **kwargs):
        super(ExpressionNode, self).__init__(**kwargs)
        self.stmts = stmts
        self.expr = expr
        self.type = expr.variable.type

    def __repr__(self):
        return "exprstat(..., %s)" % self.expr


class FunctionWrapperNode(Node):
    """
    This code is a wrapper function callable from Python using NumbaFunction
    (see numba/numbafunction.c):

        PyObject *(*)(PyObject *self, PyObject *args)

    It unpacks the tuple to native types, calls the wrapped function, and
    coerces the return type back to an object.
    """

    _fields = ['body', 'return_result']

    def __init__(self, wrapped_function, signature, orig_py_func, fake_pyfunc,
                 orig_py_func_name):
        self.wrapped_function = wrapped_function
        self.signature = signature
        self.orig_py_func = orig_py_func
        self.fake_pyfunc = fake_pyfunc
        self.name = orig_py_func_name

